@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.18 2002/04/19 12:14:15 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.20 2005/08/08 19:50:23 abe Exp abe $";
 #endif
 
 #if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
@@ -96,10 +96,10 @@ islocked(lp)
 
 	if (ety == -1) {
 
-# if	HPUXV<1030
+# if	HPUXV<1020
 	    ety = 0;
-	    ei = 0x7fffffff
-# else	/* HPUXV>=1030 */
+	    ei = 0x7fffffff;
+# else	/* HPUXV>=1020 */
 	    if (sizeof(ll.ll_end) == 4) {
 		ety = 0;
 		ei = 0x80000000;
@@ -107,8 +107,9 @@ islocked(lp)
 		ety = 1;
 		el = 0x10000000000ll;
 	    }
+# endif	/* HPUXV<1020 */
+
 	}
-# endif	/* HPUXV<1030 */
 
 /*
  * Search the locklist chain for this process.
@@ -696,22 +697,22 @@ process_node(va)
 #if	defined(HAS_AFS)
 	case N_AFS:
 	    if (an.ino_st) {
-		Lf->inode = an.inode;
+		Lf->inode = (INODETYPE)an.inode;
 		Lf->inp_ty = 1;
 	    }
 	    break;
 #endif	/* defined(HAS_AFS) */
 
 	case N_MVFS:
-	    Lf->inode = (unsigned long)m.m_ino;
+	    Lf->inode = (INODETYPE)m.m_ino;
 	    Lf->inp_ty = 1;
 	    break;
 	case N_NFS:
 
 #if	HPUXV<1030
-	    Lf->inode = (unsigned long)r.r_nfsattr.na_nodeid;
+	    Lf->inode = (INODETYPE)r.r_nfsattr.na_nodeid;
 #else	/* HPUXV>=1030 */
-	    Lf->inode = (unsigned long)r.r_attr.va_nodeid;
+	    Lf->inode = (INODETYPE)r.r_attr.va_nodeid;
 #endif	/* HPUXV<1030 */
 
 	    Lf->inp_ty = 1;
@@ -719,16 +720,16 @@ process_node(va)
 
 #if	HPUXV>=1000
 	case N_CDFS:
-	    Lf->inode = (unsigned long)c.cd_num;
+	    Lf->inode = (INODETYPE)c.cd_num;
 	    Lf->inp_ty = 1;
 	    break;
 	case N_FIFO:
 	case N_PIPE:
 	    if (vats) {
-		Lf->inode = (unsigned long)vat.va_nodeid;
+		Lf->inode = (INODETYPE)vat.va_nodeid;
 		Lf->inp_ty = 1;
 	    } else {
-		Lf->inode = (unsigned long)v->v_nodeid;
+		Lf->inode = (INODETYPE)v->v_nodeid;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -745,7 +746,7 @@ process_node(va)
 
 # if	HPUXV>=900
 	    if (rns) {
-		Lf->inode = (unsigned long)r.r_nfsattr.na_nodeid;
+		Lf->inode = (INODETYPE)r.r_nfsattr.na_nodeid;
 		Lf->inp_ty = 1;
 		break;
 	    }
@@ -758,7 +759,7 @@ process_node(va)
 	case N_REGLR:
 	case N_SPEC:
 	    if (ins) {
-		Lf->inode = (unsigned long)i.i_number;
+		Lf->inode = (INODETYPE)i.i_number;
 		Lf->inp_ty = 1;
 	    }
 	}
@@ -1018,11 +1019,7 @@ process_node(va)
 	    }
 	    break;
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = (char *)NULL;
 	}
 	if (ty)
