@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.15 2001/02/13 13:54:36 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.16 2004/01/16 05:34:48 abe Exp abe $";
 #endif
 
 #if	defined(HPUXKERNBITS)
@@ -150,20 +150,45 @@ gather_proc_info()
 	KA_T fp;
 	int err, i, j;
 
-#if	HPUXV>=1020
-struct ofile_t {
-	struct ofa {
+#if	HPUXV>=1020 && HPUXV<1100
+	struct ofile_t {
+	    struct ofa {
 		KA_T ofile;
 		int d1;
 		int pofile;
-	} ofa[SFDCHUNK];
-};
-#endif	/* HPUXV>=1020 */
+	    } ofa[SFDCHUNK];
+	};
+	struct ofa *ofap;
+	int ofasz = (int)sizeof(struct ofa);
+	char *oftp = (char *)&oft;
+	int oftsz = (int)sizeof(struct ofile_t);
+#else	/* HPUXV<1020 || HPUXV>=1100 */
+# if	HPUXV>=1100
+	struct ofa {
+	    KA_T ofile;
+	    int d1;
+	    short d2;
+	    char d3;
+	    char pofile;
+	};
+	struct ofa *ofap;
+	int ofasz = (int)sizeof(struct ofa);
+	char *oftp = (char *)NULL;
+	int oftsz = (int)(sizeof(struct ofa) * SFDCHUNK);
+	KA_T v;
+# endif	/* HPUXV>=1100 */
+#endif	/* HPUXV>=1020 && HPUXV<1100 */
 
 #if	HPUXV>=800
 	char *c, *s;
 	KA_T pfp, ofp;
+
+#if	HPUXV<1020
 	struct ofile_t oft;
+	char *oftp = (char *)&oft;
+	int oftsz = (int)sizeof(struct ofile_t);
+#endif	/* HPUXV<1020 */
+
 	struct pst_status ps;
 
 # if	HPUXV<1010
@@ -197,6 +222,38 @@ struct ofile_t {
 # endif	/* HPUXV>=1020 || (HPUXV>=900 && HPUXV<1000) */
 #endif	/* defined(HASFSTRUCT) */
 
+#if	HPUXV>=1100
+/*
+ * Define FD chunk size and pointer for HP-UX >= 11.
+ */
+	if (!oftp) {
+	    if ((get_Nl_value("chunksz", Drive_Nl, &v) >= 0) && v) {
+		if (kread(v, (char *)&oftsz, sizeof(oftsz))) {
+		    (void) fprintf(stderr, "%s: can't get FD chunk size\n",
+			Pn);
+		    Exit(1);
+		}
+		if (!oftsz) {
+		    (void) fprintf(stderr, "%s: bad FD chunk size: %d\n",
+			Pn, oftsz);
+		    Exit(1);
+		}
+	    }
+	    ofasz = (int)(oftsz / SFDCHUNK);
+	    if (oftsz != (ofasz * SFDCHUNK)) {
+		(void) fprintf(stderr,
+		    "%s: FD chunk size (%d) not exact multiple of %d\n",
+		    Pn, oftsz, SFDCHUNK);
+		Exit(1);
+	    }
+	    if (!(oftp = (char *)malloc((MALLOC_S)oftsz))) {
+		(void) fprintf(stderr, "%s: no space for %d FD bytes\n",
+		    Pn, oftsz);
+		Exit(1);
+	    }
+	}
+#endif	/* HPUXV>=1100 */
+
 /*
  * Examine proc structures and their associated information.
  */
@@ -364,22 +421,27 @@ struct ofile_t {
 
 #if	HPUXV>=800
 		if (j >= SFDCHUNK) {
-		    if (!pfp
-		    ||  kread((KA_T)pfp, (char *)&ofp, sizeof(ofp))
-		    ||  !ofp
-		    ||  kread((KA_T)ofp, (char *)&oft, sizeof(oft)))
+		    if (!pfp || kread((KA_T)pfp, (char *)&ofp, sizeof(ofp))
+		    ||  !ofp || kread((KA_T)ofp, oftp, oftsz))
 			break;
 		    j = 0;
 		    pfp += sizeof(KA_T);
+
+# if	HPUXV>=1020
+		    ofap = (struct ofa *)oftp;
+# endif	/* HPUXV>=1020 */
+
 		}
 		j++;
 
 # if	HPUXV>=1020
 #  if	defined(USESPOFILE)
-		pof = (long)oft.ofa[j - 1].pofile;
+		pof = (long)ofap->pofile;
 #  endif	/* defined(USESPOFILE) */
 
-		if ((fp = (KA_T)oft.ofa[j - 1].ofile))
+		fp = (KA_T)ofap->ofile;
+		ofap = (struct ofa *)((char *)ofap + ofasz);
+		if (fp)
 # else	/* HPUXV<1020 */
 #  if	defined(USESPOFILE)
 		pof = (long)oft.pofile[j - 1];
@@ -647,7 +709,7 @@ kread(addr, buf, len)
 {
 	int br;
 
-	if (lseek(Kd, addr, L_SET) == (off_t)-1L)
+	if (lseek(Kd, (off_t)addr, L_SET) == (off_t)-1L)
 	    return(-1);
 	br = read(Kd, buf, len);
 	return((br == len) ? 0 : 1);
