@@ -108,12 +108,12 @@ static int Nzpss = 0;			/* Nzps status: 1 = zeroed */
 	+ (int)((((((struct psfsid *)i)->psfs_type * 31415) << 5)&0xfffffff))) \
 	& (NFSIDH - 1)))
 #define	HASHPSFILEID(p)	(Nchash + \
-	(int)(((int)((((struct psfileid *)p)->psf_fsid.psfs_id * 31415) << 3) \
+	(((int)(((int)((((struct psfileid *)p)->psf_fsid.psfs_id * 31415) << 3)\
 		& 0xfffffff) \
 	+ (int)(((((struct psfileid *)p)->psf_fsid.psfs_type * 31415) << 5) \
 		& 0xfffffff) \
 	+ (int)(((((struct psfileid *)p)->psf_fileid * 31415) << 7) \
-		& 0xfffffff) \
+		& 0xfffffff))) \
 	& Ncmask))
 
 
@@ -128,6 +128,35 @@ _PROTOTYPE(static void ncache_size,(void));
 #endif	/* defined(HASNCACHE) */
 
 
+#if	defined(HASIPv6)
+/*
+ * gethostbyname2() -- an RFC2133-compatible get-host-by-name-two function
+ *                     to get AF_INET and AF_INET6 addresses from host names,
+ *                     using the gethostbyname() and RFC2553-compatible
+ *		       getipnodebyname() functions
+ */
+
+extern struct hostent *
+gethostbyname2(nm, prot)
+	char *nm; 			/* host name */
+	int prot;			/* protocol -- AF_INET or AF_INET6 */
+{
+	int err;
+
+	if (prot == AF_INET) {
+
+	/*
+	 * This shouldn't be necessary if /etc/nsswitch.conf is correct, but
+	 * it's a good fail-safe in case /etc/nsswitch.conf is missing or
+	 * incorrect.
+	 */
+	    return(gethostbyname(nm));
+	}
+	return(getipnodebyname(nm, prot, 0, &err));
+}
+#endif	/* defined(HASIPv6) */
+
+
 /*
  * get_max_fd() -- get maximum file descriptor plus one
  */
@@ -177,7 +206,7 @@ ncache_alloc()
 	if (!(Nchash = (struct l_nc **)calloc(Nceh, sizeof(struct l_nc *))))
 	{
 	    (void) fprintf(stderr,
-		"%s: can't allocate %ld local name cache entries\n", Pn, Nceh);
+		"%s: can't allocate %d local name cache entries\n", Pn, Nceh);
 	    Exit(1);
 	}
 	if (Ncfsid)
@@ -185,7 +214,7 @@ ncache_alloc()
 	if (!(Ncfsid = (struct l_fic **)calloc(NFSIDH, sizeof(struct l_fic *))))
 	{
 	    (void) fprintf(stderr,
-		"%s: can't allocate %ld local file system cache entries\n",
+		"%s: can't allocate %d local file system cache entries\n",
 		Pn, NFSIDH);
 	    Exit(1);
 	}
@@ -266,6 +295,8 @@ ncache_isroot(ps)
 void
 ncache_load()
 {
+	if (!Fncache)
+	    return;
 	(void) ncache_alloc();
 	if (!Nzpfs) {
 	    (void)memset((void *)&Nzpf, 0, sizeof(Nzpf));
@@ -291,7 +322,7 @@ ncache_loadfs(fsid, fh)
 	struct l_fic *f;
 	int i, nl, nr;
 	struct pst_mpathnode mp[DNLCINCR];
-	struct l_nc **nh, *nn, *np, *nt, *ntp;
+	struct l_nc **nh, *nn, *nt, *ntp;
 	int x = 0;
 /*
  * Allocate a new file system pointer structure and link it to its bucket.
@@ -329,8 +360,8 @@ ncache_loadfs(fsid, fh)
 		}
 		if (!(nn = (struct l_nc *)malloc(sizeof(struct l_nc)))) {
 		    (void) fprintf(stderr,
-			"%s: no name cache entry space (%d) for:%s\n",
-			Pn, sizeof(struct l_nc), mp[i].psr_name);
+			"%s: no name cache entry space (%d) for: %s\n",
+			Pn, (int)sizeof(struct l_nc), mp[i].psr_name);
 		    Exit(1);
 		}
 	    /*
@@ -398,7 +429,7 @@ ncache_lookup(buf, blen, fp)
 	int *fp;			/* full path reply */
 {
 	char *cp = buf;
-	int ef, fx, hx;
+	int ef;
 	struct l_fic **fh, *fs;
 	struct l_nc *lc;
 	int nl, rlen;
@@ -418,10 +449,11 @@ ncache_lookup(buf, blen, fp)
 # endif	/* defined(HASFSINO) */
 
 /*
- * See if cache has been loaded for this pfsid.  Don't try to load unless
- * the real or effective UID of this process is root.
+ * See if cache has been loaded for this pfsid.  Don't try to load if cache
+ * loading has been inhibited with -C, or unless the real or effective UID of
+ * this process is root.
  */
-	if (!Myuid || Setuidroot) {
+	if ((!Myuid || Setuidroot) && Fncache) {
 	    for (fh = HASHFSID(&Lf->psfid.psf_fsid), fs = *fh;
 		 fs;
 		 fs = fs->next)
@@ -451,14 +483,15 @@ ncache_lookup(buf, blen, fp)
 # endif	/* defined(HASFSINO) */
 
 	/*
-	 * If the file system's name cache couldn't be loaded -- e.g., this
-	 * lsof process lacks permission to load it -- but the UID of the
-	 * file's process matches the UID of the lsof process, see if it's
-	 * possible to read the single path name for this particular file.
-	 * (The file must have a non-zero opaque ID.)
+	 * If the file system's cache couldn't be loaded -- e.g., this lsof
+	 * process lacks permission to load it or cache lookup is inhibited
+	 * with -C -- but the UID of the file's process matches the UID of the
+	 * lsof process, see if it's possible to read the single path name for
+	 * this particular file.  (The file must have a non-zero opaque ID.)
 	 */
 	    if (!fs) {
-		if ((Myuid == Lp->uid)
+		if (Fncache
+		&&  (Myuid == Lp->uid)
 		&&  memcmp((void *)&Lf->opfid, (void *)&Nzpf, sizeof(Nzpf))
 		&&  (nl = pstat_getpathname(buf, (blen - 1), &Lf->opfid)) > 0)
 		{
@@ -474,7 +507,7 @@ ncache_lookup(buf, blen, fp)
 
 	/*
 	 * If the node is the root of the file system, return a response
-	 * that will cause the root diretory to be displayed.
+	 * that will cause the root directory to be displayed.
 	 */
 	    return(cp);
 	}
@@ -595,7 +628,8 @@ print_dev(lf, dev)
 {
 	static char buf[128];
 
-	(void) snpf(buf, sizeof(buf), "%d,%#x", major(*dev), minor(*dev));
+	(void) snpf(buf, sizeof(buf), "%d,%#x", GET_MAJ_DEV(*dev),
+		    GET_MIN_DEV(*dev));
 	return(buf);
 }
 
@@ -670,8 +704,8 @@ process_finfo(pd, opfid, psfid, na)
 	    Ntype = N_FIFO;
 	    break;
 	default:
-	    (void) snpf(buf, sizeof(buf), "%#x",
-		(pd->psfd_mode & PS_IFMT) >> 12);
+	    (void) snpf(buf, sizeof(buf), "%04o",
+		(unsigned int)(((pd->psfd_mode & PS_IFMT) >> 12) & 0xfff));
 	    cp = buf;
 	    Ntype = N_REGLR;
 	}
@@ -696,7 +730,7 @@ process_finfo(pd, opfid, psfid, na)
 /*
  * Save node number.
  */
-	Lf->inode = (unsigned long)pd->psfd_ino;
+	Lf->inode = (INODETYPE)pd->psfd_ino;
 	Lf->inp_ty = 1;
 /*
  * Save link count.
@@ -751,8 +785,10 @@ process_finfo(pd, opfid, psfid, na)
 /*
  * See if this is an NFS file.
  */
-	if (HasNFS && mp && mp->is_nfs) {
-	    if (Fnfs)
+	if (Fnfs) {
+	    if (HasNFS < 0)
+		(void) scanmnttab();
+	    if (HasNFS && mp && mp->is_nfs)
 		Lf->sf |= SELNFS;
 	}
 /*
