@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.28 2005/05/11 12:53:27 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.30 2005/08/29 10:23:33 abe Exp $";
 #endif
 
 
@@ -55,7 +55,7 @@ struct ax25sin {			/* AX25 socket information */
 	char *da;			/* destination address */
 	char *dev_ch;			/* device characters */
 	char *sa;			/* source address */
-	unsigned long inode;
+	INODETYPE inode;
 	unsigned long sq, rq;		/* send and receive queue values */
 	unsigned char sqs, rqs;		/* send and receive queue states */
 	int state;
@@ -63,7 +63,7 @@ struct ax25sin {			/* AX25 socket information */
 };
 
 struct ipxsin {				/* IPX socket information */
-	unsigned long inode;
+	INODETYPE inode;
 	char *la;			/* local address */
 	char *ra;			/* remote address */
 	int state;
@@ -72,7 +72,7 @@ struct ipxsin {				/* IPX socket information */
 };
 
 struct rawsin {				/* raw socket information */
-	unsigned long inode;
+	INODETYPE inode;
 	char *la;			/* local address */
 	char *ra;			/* remote address */
 	char *sp;			/* state characters */
@@ -84,7 +84,7 @@ struct rawsin {				/* raw socket information */
 
 struct tcp_udp {			/* IPv4 TCP and UDP socket
 					 * information */
-	unsigned long inode;
+	INODETYPE inode;
 	unsigned long faddr, laddr;	/* foreign & local IPv6 addresses */
 	int fport, lport;		/* foreign & local ports */
 	unsigned long txq, rxq;		/* transmit & receive queue values */
@@ -96,7 +96,7 @@ struct tcp_udp {			/* IPv4 TCP and UDP socket
 #if	defined(HASIPv6)
 struct tcp_udp6 {			/* IPv6 TCP and UDP socket
 					 * information */
-	unsigned long inode;
+	INODETYPE inode;
 	struct in6_addr faddr, laddr;	/* foreign and local IPv6 addresses */
 	int fport, lport;		/* foreign & local ports */
 	unsigned long txq, rxq;		/* transmit & receive queue values */
@@ -107,12 +107,12 @@ struct tcp_udp6 {			/* IPv6 TCP and UDP socket
 #endif	/* defined(HASIPv6) */
 
 struct uxsin {				/* UNIX socket information */
-	unsigned long inode;
+	INODETYPE inode;
 	char *pcb;
 	char *path;
 	unsigned char sb_def;
 	dev_t sb_dev;
-	unsigned long sb_ino;
+	INODETYPE sb_ino;
 	dev_t sb_rdev;
 	struct uxsin *next;
 };
@@ -167,11 +167,11 @@ static struct uxsin **Uxsin = (struct uxsin **)NULL;
  * Local function prototypes
  */
 
-_PROTOTYPE(static struct ax25sin *check_ax25,(unsigned long i));
-_PROTOTYPE(static struct ipxsin *check_ipx,(unsigned long i));
-_PROTOTYPE(static struct rawsin *check_raw,(unsigned long i));
-_PROTOTYPE(static struct tcp_udp *check_tcpudp,(unsigned long i, char **p));
-_PROTOTYPE(static struct uxsin *check_unix,(unsigned long i));
+_PROTOTYPE(static struct ax25sin *check_ax25,(INODETYPE i));
+_PROTOTYPE(static struct ipxsin *check_ipx,(INODETYPE i));
+_PROTOTYPE(static struct rawsin *check_raw,(INODETYPE i));
+_PROTOTYPE(static struct tcp_udp *check_tcpudp,(INODETYPE i, char **p));
+_PROTOTYPE(static struct uxsin *check_unix,(INODETYPE i));
 _PROTOTYPE(static void get_ax25,(char *p));
 _PROTOTYPE(static void get_ipx,(char *p));
 _PROTOTYPE(static void get_raw,(char *p));
@@ -179,11 +179,10 @@ _PROTOTYPE(static void get_tcpudp,(char *p, int pr, int clr));
 _PROTOTYPE(static void get_unix,(char *p));
 _PROTOTYPE(static void print_ax25info,(struct ax25sin *ap));
 _PROTOTYPE(static void print_ipxinfo,(struct ipxsin *ip));
-_PROTOTYPE(static void print_rawinfo,(struct rawsin *ip));
 
 #if	defined(HASIPv6)
-_PROTOTYPE(static struct rawsin *check_raw6,(unsigned long i));
-_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(unsigned long i, char **p));
+_PROTOTYPE(static struct rawsin *check_raw6,(INODETYPE i));
+_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(INODETYPE i, char **p));
 _PROTOTYPE(static void get_raw6,(char *p));
 _PROTOTYPE(static void get_tcpudp6,(char *p, int pr, int clr));
 _PROTOTYPE(static int net6a2in6,(char *as, struct in6_addr *ad));
@@ -197,7 +196,7 @@ _PROTOTYPE(static int net6a2in6,(char *as, struct in6_addr *ad));
 
 static struct ax25sin *
 check_ax25(i)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 {
 	struct ax25sin *ap;
 	int h;
@@ -217,7 +216,7 @@ check_ax25(i)
 
 static struct ipxsin *
 check_ipx(i)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 {
 	int h;
 	struct ipxsin *ip;
@@ -237,7 +236,7 @@ check_ipx(i)
 
 static struct rawsin *
 check_raw(i)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 {
 	int h;
 	struct rawsin *rp;
@@ -257,7 +256,7 @@ check_raw(i)
 
 static struct tcp_udp *
 check_tcpudp(i, p)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 	char **p;			/* protocol return */
 {
 	int h;
@@ -281,7 +280,7 @@ check_tcpudp(i, p)
 
 static struct rawsin *
 check_raw6(i)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 {
 	int h;
 	struct rawsin *rp;
@@ -301,7 +300,7 @@ check_raw6(i)
 
 static struct tcp_udp6 *
 check_tcpudp6(i, p)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 	char **p;			/* protocol return */
 {
 	int h;
@@ -325,7 +324,7 @@ check_tcpudp6(i, p)
 
 static struct uxsin *
 check_unix(i)
-	unsigned long i;		/* socket file's inode number */
+	INODETYPE i;			/* socket file's inode number */
 {
 	int h;
 	struct uxsin *up;
@@ -350,7 +349,7 @@ get_ax25(p)
 	struct ax25sin *ap, *np;
 	FILE *as;
 	char buf[MAXPATHLEN], *da, *dev_ch, *ep, **fp, *sa;
-	int h, i, nf;
+	int h, nf;
 	unsigned long inode, rq, sq, state;
 	MALLOC_S len;
 	unsigned char rqs, sqs;
@@ -388,7 +387,7 @@ get_ax25(p)
 	if (!(as = fopen(p, "r")))
 	    return;
 	while (fgets(buf, sizeof(buf) - 1, as)) {
-	    if ((nf = get_fields(buf, (char *)NULL, &fp)) < 24)
+	    if ((nf = get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0)) < 24)
 		continue;
 	/*
 	 * /proc/next/ac25 has no title line, a very poor deficiency in its
@@ -414,9 +413,9 @@ get_ax25(p)
 	    ||  (inode = strtoul(fp[23], &ep, 0)) == ULONG_MAX
 	    ||  !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (ap = AX25sin[h]; ap; ap = ap->next) {
-		if (inode == ap->inode)
+		if ((INODETYPE)inode == ap->inode)
 		    break;
 	    }
 	    if (ap)
@@ -497,7 +496,7 @@ get_ax25(p)
 	    }
 	    ap->da = da;
 	    ap->dev_ch = dev_ch;
-	    ap->inode = inode;
+	    ap->inode = (INODETYPE)inode;
 	    ap->rq = rq;
 	    ap->rqs = rqs;
 	    ap->sa = sa;
@@ -558,7 +557,7 @@ get_ipx(p)
 	if (!(xs = fopen(p, "r")))
 	    return;
 	while (fgets(buf, sizeof(buf) - 1, xs)) {
-	    if (get_fields(buf, (char *)NULL, &fp) < 7)
+	    if (get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0) < 7)
 		continue;
 	    if (fl) {
 
@@ -592,9 +591,9 @@ get_ipx(p)
 	    ||  (inode = strtoul(fp[6], &ep, 0)) == ULONG_MAX
 	    ||  !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (ip = Ipxsin[h]; ip; ip = ip->next) {
-		if (inode == ip->inode)
+		if ((INODETYPE)inode == ip->inode)
 		    break;
 	    }
 	    if (ip)
@@ -654,7 +653,7 @@ get_ipx(p)
 		    Pn, sizeof(struct ipxsin));
 		Exit(1);
 	    }
-	    ip->inode = inode;
+	    ip->inode = (INODETYPE)inode;
 	    ip->la = la;
 	    ip->ra = ra;
 	    ip->txq = txq;
@@ -714,7 +713,7 @@ get_raw(p)
 	if (!(xs = fopen(p, "r")))
 	    return;
 	while (fgets(buf, sizeof(buf) - 1, xs)) {
-	    if (get_fields(buf, (char *)NULL, &fp) < nf)
+	    if (get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0) < nf)
 		continue;
 	    if (nf == 12) {
 
@@ -745,9 +744,9 @@ get_raw(p)
 	    ||  (inode = strtoul(fp[9], &ep, 0)) == ULONG_MAX
 	    ||  !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (rp = Rawsin[h]; rp; rp = rp->next) {
-		if (inode == rp->inode)
+		if ((INODETYPE)inode == rp->inode)
 		    break;
 	    }
 	    if (rp)
@@ -801,7 +800,7 @@ get_raw(p)
 		    Pn, sizeof(struct rawsin));
 		Exit(1);
 	    }
-	    rp->inode = inode;
+	    rp->inode = (INODETYPE)inode;
 	    rp->la = la;
 	    rp->lal = lal;
 	    rp->ra = ra;
@@ -860,7 +859,10 @@ get_tcpudp(p, pr, clr)
 	    return;
 	nf = 12;
 	while(fgets(buf, sizeof(buf) - 1, fs)) {
-	    if (get_fields(buf, (nf == 12) ? (char *)NULL : ":", &fp) < nf)
+	    if (get_fields(buf,
+			   (nf == 12) ? (char *)NULL : ":",
+			   &fp, (int *)NULL, 0)
+	    < nf)
 		continue;
 	    if (nf == 12) {
 		if (!fp[1]  || strcmp(fp[1],  "local_address")
@@ -921,9 +923,9 @@ get_tcpudp(p, pr, clr)
 	    if (!fp[13] || !*fp[13]
 	    ||  (inode = strtoul(fp[13], &ep, 0)) == ULONG_MAX || !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (tp = TcpUdp[h]; tp; tp = tp->next) {
-		if (tp->inode == inode)
+		if (tp->inode == (INODETYPE)inode)
 		    break;
 	    }
 	    if (tp)
@@ -937,7 +939,7 @@ get_tcpudp(p, pr, clr)
 		    Pn, sizeof(struct tcp_udp));
 		Exit(1);
 	    }
-	    tp->inode = inode;
+	    tp->inode = (INODETYPE)inode;
 	    tp->faddr = faddr;
 	    tp->fport = (int)(fport & 0xffff);
 	    tp->laddr = laddr;
@@ -1001,7 +1003,7 @@ get_raw6(p)
 	if (!(xs = fopen(p, "r")))
 	    return;
 	while (fgets(buf, sizeof(buf) - 1, xs)) {
-	    if (get_fields(buf, (char *)NULL, &fp) < nf)
+	    if (get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0) < nf)
 		continue;
 	    if (nf == 12) {
 
@@ -1032,9 +1034,9 @@ get_raw6(p)
 	    ||  (inode = strtoul(fp[9], &ep, 0)) == ULONG_MAX
 	    ||  !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (rp = Rawsin6[h]; rp; rp = rp->next) {
-		if (inode == rp->inode)
+		if ((INODETYPE)inode == rp->inode)
 		    break;
 	    }
 	    if (rp)
@@ -1088,7 +1090,7 @@ get_raw6(p)
 		    Pn, sizeof(struct rawsin));
 		Exit(1);
 	    }
-	    rp->inode = inode;
+	    rp->inode = (INODETYPE)inode;
 	    rp->la = la;
 	    rp->lal = lal;
 	    rp->ra = ra;
@@ -1148,7 +1150,10 @@ get_tcpudp6(p, pr, clr)
 	    return;
 	nf = 12;
 	while(fgets(buf, sizeof(buf) - 1, fs)) {
-	    if (get_fields(buf, (nf == 12) ? (char *)NULL : ":", &fp) < nf)
+	    if (get_fields(buf,
+			   (nf == 12) ? (char *)NULL : ":",
+			   &fp, (int *)NULL, 0)
+	    < nf)
 		continue;
 	    if (nf == 12) {
 		if (!fp[1]  || strcmp(fp[1],  "local_address")
@@ -1205,9 +1210,9 @@ get_tcpudp6(p, pr, clr)
 	    if (!fp[13] || !*fp[13]
 	    ||  (inode = strtoul(fp[13], &ep, 0)) == ULONG_MAX || !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (tp6 = TcpUdp6[h]; tp6; tp6 = tp6->next) {
-		if (tp6->inode == inode)
+		if (tp6->inode == (INODETYPE)inode)
 		    break;
 	    }
 	    if (tp6)
@@ -1221,7 +1226,7 @@ get_tcpudp6(p, pr, clr)
 		    Pn, sizeof(struct tcp_udp6));
 		Exit(1);
 	    }
-	    tp6->inode = inode;
+	    tp6->inode = (INODETYPE)inode;
 	    tp6->faddr = faddr;
 	    tp6->fport = (int)(fport & 0xffff);
 	    tp6->laddr = laddr;
@@ -1281,7 +1286,7 @@ get_unix(p)
 	if (!(us = fopen(p, "r")))
 	    return;
 	while (fgets(buf, sizeof(buf) - 1, us)) {
-	    if ((nf = get_fields(buf, ":", &fp)) < 7)
+	    if ((nf = get_fields(buf, ":", &fp, (int *)NULL, 0)) < 7)
 		continue;
 	    if (fl) {
 
@@ -1316,9 +1321,9 @@ get_unix(p)
 	    if (!fp[6] || !*fp[6]
 	    ||  (inode = strtoul(fp[6], &ep, 0)) == ULONG_MAX || !ep || *ep)
 		continue;
-	    h = INOHASH(inode);
+	    h = INOHASH((INODETYPE)inode);
 	    for (up = Uxsin[h]; up; up = up->next) {
-		if (inode == up->inode)
+		if ((INODETYPE)inode == up->inode)
 		    break;
 	    }
 	    if (up)
@@ -1357,7 +1362,7 @@ get_unix(p)
 		    Pn, sizeof(struct uxsin));
 		Exit(1);
 	    }
-	    up->inode = inode;
+	    up->inode = (INODETYPE)inode;
 	    up->pcb = pcb;
 	    up->sb_def = 0;
 	    if ((up->path = path) && (*path == '/')) {
@@ -1377,7 +1382,7 @@ get_unix(p)
 		if (sr && ((sb.st_mode & S_IFMT) == S_IFSOCK)) {
 		    up->sb_def = 1;
 		    up->sb_dev = sb.st_dev;
-		    up->sb_ino = (unsigned long)sb.st_ino;
+		    up->sb_ino = (INODETYPE)sb.st_ino;
 		    up->sb_rdev = sb.st_rdev;
 		}
 	    }
@@ -1687,7 +1692,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    AX25path = (char *)NULL;
 	}
 	if ((ss & SB_INO)
-	&&  (ap = check_ax25((unsigned long)s->st_ino))
+	&&  (ap = check_ax25((INODETYPE)s->st_ino))
 	) {
 	
 	/*
@@ -1711,7 +1716,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    Ipxpath = (char *)NULL;
 	}
 	if ((ss & SB_INO)
-	&&  (ip = check_ipx((unsigned long)s->st_ino))
+	&&  (ip = check_ipx((INODETYPE)s->st_ino))
 	) {
 
 	/*
@@ -1722,7 +1727,7 @@ process_proc_sock(p, s, ss, l, lss)
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "ipx");
 	    if (ss & SB_INO) {
-		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inode = (INODETYPE)s->st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    if (ss & SB_DEV) {
@@ -1771,7 +1776,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    Rawpath = (char *)NULL;
 	}
 	if ((ss & SB_INO)
-	&&  (rp = check_raw((unsigned long)s->st_ino))
+	&&  (rp = check_raw((INODETYPE)s->st_ino))
 	) {
 
 	/*
@@ -1782,7 +1787,7 @@ process_proc_sock(p, s, ss, l, lss)
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "raw");
 	    if (ss & SB_INO) {
-		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inode = (INODETYPE)s->st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    cp = Namech;
@@ -1835,7 +1840,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    UNIXpath = (char *)NULL;
 	}
 	if ((ss & SB_INO)
-	&&  (up = check_unix((unsigned long)s->st_ino))
+	&&  (up = check_unix((INODETYPE)s->st_ino))
 	) {
 
 	/*
@@ -1850,7 +1855,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    if (up->pcb)
 		enter_dev_ch(up->pcb);
 	    if (ss & SB_INO) {
-		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inode = (INODETYPE)s->st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    path = up->path ? up->path : p;
@@ -1879,7 +1884,7 @@ process_proc_sock(p, s, ss, l, lss)
 		    unsigned char sv_inp_ty;	/* saved inp_ty */
 		    unsigned char sv_rdev_def;	/* saved rdev_def */
 		    dev_t sv_dev;		/* saved dev */
-		    unsigned long sv_inode;	/* saved inode */
+		    INODETYPE sv_inode;		/* saved inode */
 		    dev_t sv_rdev;		/* saved rdev */
 
 		    sv_dev_def = Lf->dev_def;
@@ -1927,7 +1932,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    Raw6path = (char *)NULL;
 	}
 	if (!Fxopt && (ss & SB_INO)
-	&&  (rp = check_raw6((unsigned long)s->st_ino))
+	&&  (rp = check_raw6((INODETYPE)s->st_ino))
 	) {
 
 	/*
@@ -1938,7 +1943,7 @@ process_proc_sock(p, s, ss, l, lss)
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "raw6");
 	    if (ss & SB_INO) {
-		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inode = (INODETYPE)s->st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    cp = Namech;
@@ -1998,7 +2003,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    UDP6path = (char *)NULL;
 	}
 	if (!Fxopt && (ss & SB_INO)
-	&&  (tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))
+	&&  (tp6 = check_tcpudp6((INODETYPE)s->st_ino, &pr))
 	) {
 
 	/*
@@ -2015,8 +2020,8 @@ process_proc_sock(p, s, ss, l, lss)
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
 	    if (ss && SB_INO) {
-		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
-		    (unsigned long)s->st_ino);
+		(void) snpf(dev_ch, sizeof(dev_ch), InodeFmt_d,
+		    (INODETYPE)s->st_ino);
 		enter_dev_ch(dev_ch);
 	    }
 	    af = AF_INET6;
@@ -2063,7 +2068,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    UDPpath = (char *)NULL;
 	}
 	if (!Fxopt && (ss & SB_INO)
-	&&  (tp = check_tcpudp((unsigned long)s->st_ino, &pr))
+	&&  (tp = check_tcpudp((INODETYPE)s->st_ino, &pr))
 	) {
 
 	/*
@@ -2086,8 +2091,8 @@ process_proc_sock(p, s, ss, l, lss)
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
 	    if (ss & SB_INO) {
-		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
-		    (unsigned long)s->st_ino);
+		(void) snpf(dev_ch, sizeof(dev_ch), InodeFmt_d,
+		    (INODETYPE)s->st_ino);
 		enter_dev_ch(dev_ch);
 	    }
 	    if (tp->faddr || tp->fport) {
@@ -2117,7 +2122,7 @@ process_proc_sock(p, s, ss, l, lss)
  */
 	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 	if (ss & SB_INO) {
-	    Lf->inode = (unsigned long)s->st_ino;
+	    Lf->inode = (INODETYPE)s->st_ino;
 	    Lf->inp_ty = 1;
 	}
 	if (ss & SB_DEV) {
