@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.34 2004/12/30 18:42:24 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.37 2006/04/02 04:33:06 abe Exp $";
 #endif
 
 
@@ -40,7 +40,7 @@ static char *rcsid = "$Id: dnode.c,v 1.34 2004/12/30 18:42:24 abe Exp $";
 
 
 #if	defined(HASFDESCFS) && HASFDESCFS==1
-_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
+_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, INODETYPE *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 #if	defined(HASPROCFS)
@@ -50,6 +50,17 @@ _PROTOTYPE(static void getmemsz,(pid_t pid));
 #define	PGSHIFT	pgshift
 # endif	/* !defined(PGSHIFT) */
 
+#if	defined(HAS_DINODE_U)
+#define	DINODE_U	dinode_u
+#else	/* !defined(HAS_DINODE_U) */
+#define	DINODE_U	i_din
+#endif	/* defined(HAS_DINODE_U) */
+
+#if	defined(HAS_UM_UFS)
+#define	UFS1	UM_UFS1
+#define	UFS2	UM_UFS2
+#endif	/* defined(HAS_UM_UFS) */
+
 
 /*
  * getmemsz() - get memory size of a /proc/<n>/mem entry
@@ -98,7 +109,7 @@ getmemsz(pid)
 static int
 lkup_dev_tty(dr, ir)
 	dev_t *dr;			/* place to return device number */
-	unsigned long *ir;		/* place to return inode number */
+	INODETYPE *ir;			/* place to return inode number */
 {
 	int i;
 
@@ -119,7 +130,7 @@ lkup_dev_tty_again:
 # endif	/* defined(HASDCACHE) */
 
 		*dr = Devtp[i].rdev;
-		*ir = (unsigned long)Devtp[i].inode;
+		*ir = Devtp[i].inode;
 		return(1);
 	    }
 	}
@@ -195,9 +206,10 @@ process_node(va)
 
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
-	unsigned long iso_ino, iso_sz;
+	INODETYPE iso_ino;
 	long iso_nlink;
 	int iso_stat;
+	SZOFFTYPE iso_sz;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASFDESCFS)
@@ -205,7 +217,7 @@ process_node(va)
 
 # if	HASFDESCFS==1
 	static dev_t f_tty_dev;
-	static unsigned long f_tty_ino;
+	static INODETYPE f_tty_ino;
 	static int f_tty_s = 0;
 # endif	/* HASFDESCFS==1 */
 
@@ -239,7 +251,7 @@ process_node(va)
 #if	defined(HASMSDOSFS)
 	struct denode d;
 	u_long dpb;
-	u_long nn;
+	INODETYPE nn;
 	struct msdosfsmount pm;
 #endif	/* defined(HASMSDOSFS) */
 
@@ -369,8 +381,7 @@ process_overlaid_node:
 
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
-	    if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
-		  &iso_nlink, (SZOFFTYPE *)&iso_sz))
+	    if (read_iso_node(v, &iso_dev, &iso_ino, &iso_nlink, &iso_sz))
 	    {
 		(void) snpf(Namech, Namechl, "can't read iso_node at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
@@ -440,7 +451,7 @@ process_overlaid_node:
 	 */
 	    if (v->v_type == VDIR && (v->v_flag & VROOT)) {
 		(void) snpf(Namech, Namechl, "%s", _PATH_KERNFS);
-		ksb.st_ino = 2;
+		ksb.st_ino = (ino_t)2;
 		ksb.st_size = DEV_BSIZE;
 		ksbs = 1;
 	    } else if (Namech[0] && statsafely(Namech, &ksb) == 0)
@@ -563,12 +574,12 @@ process_overlaid_node:
 		nty = EXT2NODE;
 
 # if	defined(HASI_E2FS_PTR)
-		if (i.i_din.e2fs_din
-		&&  !kread((KA_T)i.i_din.e2fs_din, (char *)&ed, sizeof(ed)))
+		if (i.DINODE_U.e2fs_din
+		&&  !kread((KA_T)i.DINODE_U.e2fs_din, (char *)&ed, sizeof(ed)))
 		    edp = &ed;
 # else	/* !defined(HASI_E2FS_PTR) */
 #  if	HASEXT2FS<2
-		edp = &i.i_din.e2fs_din;
+		edp = &i.DINODE_U.e2fs_din;
 #  else	/* HASEXT2FS>=2 */
 		edp = &i.i_e2din;
 #  endif	/* HASEXT2FS>=2 */
@@ -588,16 +599,16 @@ process_overlaid_node:
 		    &&  !kread((KA_T)i.i_ump, (char *)&um, sizeof(um))) {
 			if (um.um_fstype == UFS1) {
 			    ffs = 1;
-			    if (i.i_din.ffs1_din
-			    &&  !kread((KA_T)i.i_din.ffs1_din, (char *)&u1,
+			    if (i.DINODE_U.ffs1_din
+			    &&  !kread((KA_T)i.DINODE_U.ffs1_din, (char *)&u1,
 				       sizeof(u1)))
 			    {
 				u1s = 1;
 			    }
 			} else if (um.um_fstype == UFS2) {
 			    ffs = 2;
-			    if (i.i_din.ffs2_din
-			    &&  !kread((KA_T)i.i_din.ffs2_din, (char *)&u2,
+			    if (i.DINODE_U.ffs2_din
+			    &&  !kread((KA_T)i.DINODE_U.ffs2_din, (char *)&u2,
 				       sizeof(u2)))
 			    {
 				u2s = 1;
@@ -719,7 +730,7 @@ process_overlaid_node:
 		}
 # else	/* !defined(HASI_E2FS_PTR) */
 #  if	HASEXT2FS<2
-		rdev = i.i_din.e2fs_din.e2di_rdev;
+		rdev = i.DINODE_U.e2fs_din.e2di_rdev;
 #  else	/* HASEXT2FS>=2 */
 		rdev = i.i_e2din.e2di_rdev;
 #  endif	/* HASEXT2FS>=2 */
@@ -787,6 +798,7 @@ process_overlaid_node:
 	case NFSNODE:
 	    dev = NVATTR.va_fsid;
 	    devs = 1;
+	    break;
 
 #if	defined(HASPTYFS)
 	case PTYFSNODE:
@@ -796,7 +808,13 @@ process_overlaid_node:
 		rdevs = 1;
 	    }
 	    if (vfs) {
+
+# if	defined(HASSTATVFS)
 		dev = (dev_t)vfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
+		dev = (dev_t)vfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
 		devs = 1;
 	    }
 	    break;
@@ -814,17 +832,17 @@ process_overlaid_node:
 		dpb = (u_long)(pm.pm_BytesPerSec / sizeof(struct direntry));
 		if (d.de_Attributes & ATTR_DIRECTORY) {
 		    if (d.de_StartCluster == MSDOSFSROOT)
-			nn = (u_long)1;
+			nn = (INODETYPE)1;
 		    else
-			nn = (u_long)(cntobn(&pm, d.de_StartCluster) * dpb);
+			nn = (INODETYPE)(cntobn(&pm, d.de_StartCluster) * dpb);
 		} else {
 		    if (d.de_dirclust == MSDOSFSROOT)
-			nn = (u_long)(roottobn(&pm, 0) * dpb);
+			nn = (INODETYPE)(roottobn(&pm, 0) * dpb);
 		    else
-			nn = (u_long)(cntobn(&pm, d.de_dirclust) * dpb);
-		    nn += (u_long)(d.de_diroffset / sizeof(struct direntry));
+			nn = (INODETYPE)(cntobn(&pm, d.de_dirclust) * dpb);
+		    nn += (INODETYPE)(d.de_diroffset / sizeof(struct direntry));
 		}
-		Lf->inode = (unsigned long)nn;
+		Lf->inode = nn;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -835,14 +853,14 @@ process_overlaid_node:
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-	    Lf->inode = (unsigned long)i.i_number;
+	    Lf->inode = (INODETYPE)i.i_number;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
 	    if (ksbs) {
-		Lf->inode = (unsigned long)ksb.st_ino;
+		Lf->inode = (INODETYPE)ksb.st_ino;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -858,13 +876,13 @@ process_overlaid_node:
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-	    Lf->inode = (unsigned long)NVATTR.va_fileid;
+	    Lf->inode = (INODETYPE)NVATTR.va_fileid;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASPROCFS)
 	case PFSNODE:
-	    Lf->inode = (unsigned long)p.pfs_fileno;
+	    Lf->inode = (INODETYPE)p.pfs_fileno;
 	    Lf->inp_ty = 1;
 	    break;
 #endif	/* defined(HASPROCFS) */
@@ -873,11 +891,11 @@ process_overlaid_node:
 	case PTYFSNODE:
 	    if (pt.ptyfs_type == PTYFSptc) {
 		if (pt.ptyfs_fileno > 0x3fffffff)
-		    Lf->inode = (unsigned long)(pt.ptyfs_fileno & 0x3fffffff);
+		    Lf->inode = (INODETYPE)(pt.ptyfs_fileno & 0x3fffffff);
 		else
-		    Lf->inode = (unsigned long)(pt.ptyfs_fileno - 1);
+		    Lf->inode = (INODETYPE)(pt.ptyfs_fileno - 1);
 	    } else
-		Lf->inode = (unsigned long)pt.ptyfs_fileno;
+		Lf->inode = (INODETYPE)pt.ptyfs_fileno;
 	    Lf->inp_ty = 1;
 	    break;
 #endif	/* defined(HASPTYFS) */
@@ -1181,11 +1199,7 @@ process_overlaid_node:
 	    ty = "FIFO";
 	    break;
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = NULL;
 	}
 	if (ty)
@@ -1353,7 +1367,7 @@ process_overlaid_node:
 		    if ((pfi->pid && pfi->pid == p.pfs_pid)
 		    
 # if	defined(HASPINODEN)
-		    ||  (Lf->inp_ty == 1 && pfi->inode == Lf->inode)
+		    ||  ((Lf->inp_ty == 1) && (pfi->inode == Lf->inode))
 # endif	/* defined(HASPINODEN) */
 
 		    ) {
