@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.10 99/06/22 08:22:02 abe Exp $
+ * $Id: dlsof.h,v 1.13 2006/03/28 22:09:23 abe Exp $
  */
 
 
@@ -165,6 +165,7 @@ typedef	off_t		KA_T;
 # endif	/* !defined(MAXPATHLEN) */
 
 #define MAXSEGS		100	/* maximum text segments */
+#define	MAXSYSCMDL	(PSCOMSIZ - 1)	/* max system command name length */
 
 # if	OSRV<500
 #define	N_UNIX		"/unix"
@@ -196,13 +197,6 @@ extern int Hz;
 extern int Kd;
 extern KA_T Lbolt;
 
-struct l_ino {
-	dev_t dev;			/* device */
-	dev_t rdev;			/* real device */
-	ino_t number;			/* inode number */
-	long size;			/* file size */
-};
-
 extern int nxdevmaps;			/* maximum kernel xdevmap[] index */
 
 struct mounts {
@@ -213,7 +207,7 @@ struct mounts {
 					 * (symbolic links resolved) */
 	dev_t dev;			/* directory st_dev */
 	dev_t rdev;			/* directory st_rdev */
-	ino_t inode;			/* directory st_ino */
+	INODETYPE inode;		/* directory st_ino */
 	mode_t mode;			/* directory st_mode */
 	mode_t fs_mode;			/* file system st_mode */
 	struct mounts *next;		/* forward link */
@@ -231,10 +225,11 @@ struct sfile {
 	char *name;			/* file name (after readlink()) */
 	char *devnm;			/* device name (optional) */
 	dev_t dev;			/* device */
+	dev_t rdev;			/* raw device */
 	mode_t mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
-	ino_t i;			/* inode number */
+	INODETYPE i;			/* inode number */
 	int f;				/* file found flag */
 	struct sfile *next;		/* forward link */
 };
