@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.49 2004/12/30 18:42:59 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.52 2006/03/27 22:57:33 abe Exp $";
 #endif
 
 
@@ -88,6 +88,8 @@ _PROTOTYPE(static int read_npn,(KA_T na, KA_T pa, struct pcnode *p));
 _PROTOTYPE(static int read_nrn,(KA_T na, KA_T ra, struct rnode *r));
 
 #if	solaris>=100000
+_PROTOTYPE(static int read_nctfsn,(int ty, KA_T na, KA_T ca, char *cn));
+_PROTOTYPE(static int read_nprtn,(KA_T na, KA_T ra, port_t *p));
 _PROTOTYPE(static int read_nrn4,(KA_T na, KA_T ra, struct rnode4 *r));
 #endif	/* solaris>=100000 */
 
@@ -122,6 +124,17 @@ _PROTOTYPE(static int read_ndvn,(KA_T na, KA_T da, struct dv_node *dv,
 
 static KA_T Auvops;		/* auto_vnodeops address */
 static KA_T Avops;		/* afs_vnodeops address */
+static KA_T Ctfsadir;		/* CTFS all directory ops address */
+static KA_T Ctfsbund;		/* CTFS bundle and pbundle ops address */
+static KA_T Ctfscdir;		/* CTFS contract ops address */
+static KA_T Ctfsctl;		/* CTFS ctl ops address */
+static KA_T Ctfsevt;		/* CTFS events ops address */
+static KA_T Ctfslate;		/* CTFS latest ops address */
+static KA_T Ctfsroot;		/* CTFS root ops address */
+static KA_T Ctfsstat;		/* CTFS status ops address */
+static KA_T Ctfssym;		/* CTFS symlink ops address */
+static KA_T Ctfstdir;		/* CTFS type ops address */
+static KA_T Ctfstmpl;		/* CTFS template ops address */
 static KA_T Cvops;		/* cache_vnodeops address */
 static KA_T Dvops;		/* door_vnodeops address */
 static KA_T Devops;		/* dv_vnodeops address */
@@ -137,9 +150,12 @@ static KA_T Nmvops;		/* nm_vnodeops address */
 static KA_T Nvops;		/* nfs_vnodeops address */
 static KA_T Pdvops;		/* pcfs_dvnodeops address */
 static KA_T Pfvops;		/* pcfs_fvnodeops address */
+static KA_T Portvops;		/* port_vnodeops address */
 static KA_T Prvops;		/* prvnodeops address */
 static KA_T Sam1vops;		/* samfs_vnodeops address */
 static KA_T Sam2vops;		/* samfs_client_vnodeops address */
+static KA_T Sam3vops;		/* samfs_vnodeopsp address */
+static KA_T Sam4vops;		/* samfs_client_vnodeopsp address */
 static KA_T Sckvops;		/* sock_vnodeops address */
 static KA_T Sncavops;		/* socknca_vnodeops address */
 static KA_T Stpivops;		/* socktpi_vnodeops address */
@@ -147,9 +163,11 @@ static KA_T Spvops;		/* spec_vnodeops address */
 static KA_T Tvops;		/* tmp_vnodeops address */
 static KA_T Uvops;		/* ufs_vnodeops address */
 static KA_T Vvops[VXVOP_NUM];	/* addresses of:
+				 *   vx_fcl_dnodeops_p (VXVOP_FCL)
+				 *   fdd_vnops (VXVOP_FDD)
 				 *   fdd_chain_vnops (VXVOP_FDDCH),
-				 *   fdd_vnops (VXVOP_FDD), and
-				 *   vx_vnodeops (VXVOP_REG) */
+				 *   vx_vnodeops (VXVOP_REG)
+				 *   vx_vnodeops_p (VXVOP_REG_P) */
 
 /*
  * Local macros
@@ -183,7 +201,6 @@ ent_fa(a1, a2, d, len)
 {
 	static char buf[1024];
 	size_t bufl = sizeof(buf);
-	MALLOC_S l, nl;
 	char tbuf[32];
 /*
  * Form the fattach description.
@@ -224,7 +241,6 @@ is_socket(v)
 	struct vnode *v;		/* vnode pointer */
 {
 	char *cp, *ep, *pf;
-	struct l_dev *dp;
 	int i, j, len, n, pfl;
 	static struct tcpudp {
 	    int ds;
@@ -505,9 +521,8 @@ static int
 idoorkeep(d)
 	struct door_node *d;		/* door's node */
 {
-	char buf[1024], *cp;
+	char buf[1024];
 	size_t bufl = sizeof(buf);
-	MALLOC_S l, nl;
 	struct proc dp;
 	struct pid dpid;
 /*
@@ -564,7 +579,6 @@ process_node(va)
 	struct fifonode f;
 	char *fa = (char *)NULL;
 	int fal;
-	struct vfs favfs;
 	static int ft = 1;
 	struct vnode fv, rv;
 	int fx;
@@ -574,7 +588,6 @@ process_node(va)
 	struct vfs kv;
 	int len, llc, nl, snl, sepl;
 	struct mvfsnode m;
-	struct mounts *mp;
 	struct namenode nn;
 	struct l_vfs *nvfs, *vfs;
 	struct pcnode pc;
@@ -618,10 +631,23 @@ process_node(va)
 	struct so_so soso;
 	int so_st = 0;
 #else	/* solaris>=100000 */
+	union {
+	    ctfs_adirnode_t adir;
+	    ctfs_bunode_t bun;
+	    ctfs_cdirnode_t cdir;
+	    ctfs_ctlnode_t ctl;
+	    ctfs_evnode_t ev;
+	    ctfs_latenode_t late;
+	    ctfs_rootnode_t root;
+	    ctfs_symnode_t sym;
+	    ctfs_tdirnode_t tdir;
+	    ctfs_tmplnode_t tmpl;
+	} ctfs;
 	dev_t dv_dev;
 	struct dv_node dv;
 	unsigned char dv_devs = 0;
 	unsigned char dvs = 0;
+	port_t pn;
 	struct rnode4 r4;
 #endif	/* solaris<100000 */
 
@@ -652,6 +678,17 @@ process_node(va)
 	    GETVOPS("avops", Drive_Nl, Avops);
 	    if (!Avops)
 		GETVOPS("Avops", Drive_Nl, Avops);
+	    GETVOPS("ctfsadir", Drive_Nl, Ctfsadir);
+	    GETVOPS("ctfsbund", Drive_Nl, Ctfsbund);
+	    GETVOPS("ctfscdir", Drive_Nl, Ctfscdir);
+	    GETVOPS("ctfsctl", Drive_Nl, Ctfsctl);
+	    GETVOPS("ctfsevt", Drive_Nl, Ctfsevt);
+	    GETVOPS("ctfslate", Drive_Nl, Ctfslate);
+	    GETVOPS("ctfsroot", Drive_Nl, Ctfsroot);
+	    GETVOPS("ctfsstat", Drive_Nl, Ctfsstat);
+	    GETVOPS("ctfssym", Drive_Nl, Ctfssym);
+	    GETVOPS("ctfstdir", Drive_Nl, Ctfstdir);
+	    GETVOPS("ctfstmpl", Drive_Nl, Ctfstmpl);
 	    GETVOPS("cvops", Drive_Nl, Cvops);
 	    GETVOPS("devops", Drive_Nl, Devops);
 	    GETVOPS("dvops", Drive_Nl, Dvops);
@@ -667,21 +704,28 @@ process_node(va)
 	    GETVOPS("nvops", Drive_Nl, Nvops);
 	    GETVOPS("pdvops", Drive_Nl, Pdvops);
 	    GETVOPS("pfvops", Drive_Nl, Pfvops);
+	    GETVOPS("portvops", Drive_Nl, Portvops);
 	    GETVOPS("prvops", Drive_Nl, Prvops);
 	    GETVOPS("sam1vops", Drive_Nl, Sam1vops);
 	    GETVOPS("sam2vops", Drive_Nl, Sam2vops);
+	    GETVOPS("sam3vops", Drive_Nl, Sam3vops);
+	    GETVOPS("sam4vops", Drive_Nl, Sam4vops);
 	    GETVOPS("sckvops", Drive_Nl, Sckvops);
 	    GETVOPS("sncavops", Drive_Nl, Sncavops);
 	    GETVOPS("stpivops", Drive_Nl, Stpivops);
 	    GETVOPS("spvops", Drive_Nl, Spvops);
 	    GETVOPS("tvops", Drive_Nl, Tvops);
 	    GETVOPS("uvops", Drive_Nl, Uvops);
+	    if (VXVOP_FCL < VXVOP_NUM)
+		GETVOPS("vvfclops", Drive_Nl, Vvops[VXVOP_FCL]);
 	    if (VXVOP_FDD < VXVOP_NUM)
 		GETVOPS("vvfops", Drive_Nl, Vvops[VXVOP_FDD]);
 	    if (VXVOP_FDDCH < VXVOP_NUM)
 		GETVOPS("vvfcops", Drive_Nl, Vvops[VXVOP_FDDCH]);
 	    if (VXVOP_REG < VXVOP_NUM)
 		GETVOPS("vvops", Drive_Nl, Vvops[VXVOP_REG]);
+	    if (VXVOP_REG_P < VXVOP_NUM)
+		GETVOPS("vvops_p", Drive_Nl, Vvops[VXVOP_REG_P]);
 	    ft = 0;
 	}
 /*
@@ -940,6 +984,23 @@ vfs_read_error:
 		return;
 	    break;
 
+#if	solaris>=100000
+	case N_CTFSADIR:
+	case N_CTFSBUND:
+	case N_CTFSCDIR:
+	case N_CTFSCTL:
+	case N_CTFSEVT:
+	case N_CTFSLATE:
+	case N_CTFSROOT:
+	case N_CTFSSTAT:
+	case N_CTFSSYM:
+	case N_CTFSTDIR:
+	case N_CTFSTMPL:
+	    if (read_nctfsn(Ntype, va, (KA_T)v->v_data, (char *)&ctfs))
+		return;
+	    break;
+#endif	/* solaris>=100000 */
+
 #if	solaris>=20600
 	case N_SOCK:
 	    if (read_nson(va, (KA_T)v->v_data, &so))
@@ -1004,8 +1065,7 @@ vfs_read_error:
 		realvp = (KA_T)f.fn_realvp;
 	    }
 	    if (!realvp) {
-		Lf->inode = (unsigned long)(nns ? nn.nm_vattr.va_nodeid
-						: f.fn_ino);
+		Lf->inode = (INODETYPE)(nns ? nn.nm_vattr.va_nodeid : f.fn_ino);
 
 #if	solaris>=80000	/* Solaris 8 and above hack! */
 # if	defined(_LP64)
@@ -1082,6 +1142,13 @@ vfs_read_error:
 		return;
 	    break;
 
+#if	solaris>=100000
+	case N_PORT:
+	    if (read_nprtn(va, (KA_T)v->v_data, &pn))
+		return;
+	    break;
+#endif	/* solaris>=100000 */
+
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    if (read_npi(va, v, &pids))
@@ -1229,6 +1296,23 @@ vfs_read_error:
 		    return;
 		break;
 
+#if	solaris>=100000
+	case N_CTFSADIR:
+	case N_CTFSBUND:
+	case N_CTFSCDIR:
+	case N_CTFSCTL:
+	case N_CTFSEVT:
+	case N_CTFSLATE:
+	case N_CTFSROOT:
+	case N_CTFSSTAT:
+	case N_CTFSSYM:
+	case N_CTFSTDIR:
+	case N_CTFSTMPL:
+	    if (read_nctfsn(vty, va, (KA_T)v->v_data, (char *)&ctfs))
+		return;
+	    break;
+#endif	/* solaris>=100000 */
+
 #if	solaris>=20600
 	    case N_SOCK:
 		if (read_nson(va, (KA_T)v->v_data, &so))
@@ -1264,6 +1348,14 @@ vfs_read_error:
 		    return;
 		nns = 1;
 		break;
+
+#if	solaris>=100000
+	    case N_PORT:
+		if (read_nprtn(va, (KA_T)v->v_data, &pn))
+		    return;
+		break;
+#endif	/* solaris>=100000 */
+
 	    case N_PCFS:
 		if (read_npn(va, (KA_T)v->v_data, &pc))
 		    return;
@@ -1372,6 +1464,26 @@ vfs_read_error:
 	    }
 	    break;
 
+#if	solaris>=100000
+	case N_CTFSADIR:
+	case N_CTFSBUND:
+	case N_CTFSCDIR:
+	case N_CTFSCTL:
+	case N_CTFSEVT:
+	case N_CTFSLATE:
+	case N_CTFSROOT:
+	case N_CTFSSTAT:
+	case N_CTFSSYM:
+	case N_CTFSTDIR:
+	case N_CTFSTMPL:
+	    if (kvs) {
+		dev = kv.vfs_dev;
+		devs = 1;
+	    }
+	    break;
+#endif	/* solaris>=100000 */
+
+
 	case N_FD:
 	    if (kvs) {
 		dev = kv.vfs_dev;
@@ -1417,7 +1529,7 @@ vfs_read_error:
 
 		    trdev = sdp->rdev;
 		    devs = rdevs = trdevs = 1;
-		    Lf->inode = (unsigned long)sdp->inode;
+		    Lf->inode = (INODETYPE)sdp->inode;
 		    Lf->inp_ty = 1;
 		    (void) snpf(Namech, Namechl, "%s", sdp->name);
 		} else
@@ -1453,11 +1565,11 @@ vfs_read_error:
 		}
 		if (nl == snl
 
-#if	solaris<90000
-		&&  so.so_ux_laddr.sou_magic == SOU_MAGIC_IMPLICIT
-#else	/* solaris>=90000 */
+#if	defined(HASSOUXSOUA)
 		&&  so.so_ux_laddr.soua_magic == SOU_MAGIC_IMPLICIT
-#endif	/* solaris<90000 */
+#else	/* !defined(HASSOUXSOUA) */
+		&&  so.so_ux_laddr.sou_magic == SOU_MAGIC_IMPLICIT
+#endif	/* defined(HASSOUXSOUA) */
 
 		) {
 
@@ -1468,11 +1580,11 @@ vfs_read_error:
 		    pa = (struct pairaddr *)&ua;
 		    if (!(peer = (KA_T)((int)pa->p)))
 
-#if	solaris<90000
-			peer = (KA_T)so.so_ux_laddr.sou_vp;
-#else	/* solaris>=90000 */
+#if	defined(HASSOUXSOUA)
 			peer = (KA_T)so.so_ux_laddr.soua_vp;
-#endif	/* solaris<90000 */
+#else	/* !defined(HASSOUXSOUA) */
+			peer = (KA_T)so.so_ux_laddr.sou_vp;
+#endif	/* defined(HASSOUXSOUA) */
 
 		    if (peer)
 			(void) snpf(ubuf, sizeof(ubuf), "(socketpair: %s)",
@@ -1494,11 +1606,11 @@ vfs_read_error:
 	     * the name.
 	     */
 
-#if	solaris<90000
-		soa = (KA_T)so.so_ux_faddr.sou_vp;
-#else	/* solaris>=90000 */
+#if	defined(HASSOUXSOUA)
 		soa = (KA_T)so.so_ux_faddr.soua_vp;
-#endif	/* solaris<90000 */
+#else	/* !defined(HASSOUXSOUA) */
+		soa = (KA_T)so.so_ux_faddr.sou_vp;
+#endif	/* defined(HASSOUXSOUA) */
 
 		(void) snpf(ubuf, sizeof(ubuf), "%s(%s%s%s)",
 		    Namech[0] ? " " : "",
@@ -1567,6 +1679,16 @@ vfs_read_error:
 		enter_dev_ch("    NMFS");
 	    break;
 
+#if	solaris>=100000
+	case N_PORT:
+	    if (kvs) {
+		dev = kv.vfs_dev;
+		devs = 1;
+	    }
+	    break;
+#endif	/* solaris>=100000 */
+
+
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    if (kvs) {
@@ -1600,7 +1722,7 @@ vfs_read_error:
 			}
 			rdev = sdp->rdev;
 			rdevs = 1;
-			Lf->inode = (unsigned long)sdp->inode;
+			Lf->inode = (INODETYPE)sdp->inode;
 			Lf->inp_ty = 1;
 			(void) snpf(ubuf, sizeof(ubuf), "(%s%s%s)",
 			    print_kptr(so_ad[0], (char *)NULL, 0),
@@ -1731,7 +1853,7 @@ vfs_read_error:
 #if	defined(HAS_AFS)
 	case N_AFS:
 	    if (an.ino_st) {
-		Lf->inode = an.inode;
+		Lf->inode = (INODETYPE)an.inode;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -1741,9 +1863,9 @@ vfs_read_error:
 	case N_AUTO:
 
 # if	solaris<20600
-	    Lf->inode = (unsigned long)au.an_nodeid;
+	    Lf->inode = (INODETYPE)au.an_nodeid;
 # else	/* solaris>=20600 */
-	    Lf->inode = (unsigned long)fnn.fn_nodeid;
+	    Lf->inode = (INODETYPE)fnn.fn_nodeid;
 # endif	/* solaris<20600 */
 
 	    Lf->inp_ty = 1;
@@ -1752,37 +1874,54 @@ vfs_read_error:
 # if	solaris>=100000
 	case N_DEV:
 	    if (dvs) {
-		Lf->inode = (unsigned long)dv.dv_ino;
+		Lf->inode = (INODETYPE)dv.dv_ino;
 		Lf->inp_ty = 1;
 	    }
 	    break;
 # endif	/* solaris>=100000 */
 
 	case N_DOOR:
-	    if (nns && (Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid)) {
+	    if (nns && (Lf->inode = (INODETYPE)nn.nm_vattr.va_nodeid)) {
 		Lf->inp_ty = 1;
 		break;
 	    }
 	    if (dns) {
-		if ((Lf->inode = (unsigned long)dn.door_index)) 
+		if ((Lf->inode = (INODETYPE)dn.door_index)) 
 		    Lf->inp_ty = 1;
 	    }
 	    break;
 #endif	/* solaris>=20500 */
 
 	case N_CACHE:
-	    Lf->inode = (unsigned long)cn.c_fileno;
+	    Lf->inode = (INODETYPE)cn.c_fileno;
 	    Lf->inp_ty = 1;
 	    break;
+
+#if	solaris>=100000
+	case N_CTFSADIR:
+	case N_CTFSBUND:
+	case N_CTFSCDIR:
+	case N_CTFSCTL:
+	case N_CTFSEVT:
+	case N_CTFSLATE:
+	case N_CTFSROOT:
+	case N_CTFSSTAT:
+	case N_CTFSSYM:
+	case N_CTFSTDIR:
+	case N_CTFSTMPL:
+	    /* Method of computing CTFS inode not known. */
+	    break;
+#endif	/* solaris>=10000 */
+
 	case N_FD:
 	    if (v->v_type == VDIR)
-		Lf->inode = (unsigned long)2;
+		Lf->inode = (INODETYPE)2;
 	    else
-		Lf->inode = GET_MIN_DEV(v->v_rdev) * 100;
+		Lf->inode = (INODETYPE)(GET_MIN_DEV(v->v_rdev) * 100);
 	    Lf->inp_ty = 1;
 	    break;
 	case N_HSFS:
-	    Lf->inode = (unsigned long)h.hs_nodeid;
+	    Lf->inode = (INODETYPE)h.hs_nodeid;
 	    Lf->inp_ty = 1;
 	    break;
 
@@ -1797,23 +1936,23 @@ vfs_read_error:
 
 	    break;
 	case N_MVFS:
-	    Lf->inode = (unsigned long)m.m_ino;
+	    Lf->inode = (INODETYPE)m.m_ino;
 	    Lf->inp_ty = 1;
 	    break;
 	case N_NFS:
-	    Lf->inode = (unsigned long)r.r_attr.va_nodeid;
+	    Lf->inode = (INODETYPE)r.r_attr.va_nodeid;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	solaris>=100000
 	case N_NFS4:
-	    Lf->inode = (unsigned long)r4.r_attr.va_nodeid;
+	    Lf->inode = (INODETYPE)r4.r_attr.va_nodeid;
 	    Lf->inp_ty = 1;
 	    break;
 #endif	/* solaris>=100000 */
 
 	case N_NM:
-	    Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid;
+	    Lf->inode = (INODETYPE)nn.nm_vattr.va_nodeid;
 	    Lf->inp_ty = 1;
 	    break;
 
@@ -1832,7 +1971,7 @@ vfs_read_error:
 	    && !kread((KA_T)kv.vfs_data, (char *)&pcfs, sizeof(pcfs))) {
 
 #if	solaris>=70000
-		Lf->inode = (unsigned long)pc_makenodeid(pc.pc_eblkno,
+		Lf->inode = (INODETYPE)pc_makenodeid(pc.pc_eblkno,
 			    pc.pc_eoffset,
 			    pc.pc_entry.pcd_attr,
 			    IS_FAT32(&pcfs)
@@ -1841,7 +1980,7 @@ vfs_read_error:
 				: ltohs(pc.pc_entry.pcd_scluster_lo),
 			    pcfs.pcfs_entps);
 #else	/* solaris<70000 */
-		Lf->inode = (unsigned long)pc_makenodeid(pc.pc_eblkno,
+		Lf->inode = (INODETYPE)pc_makenodeid(pc.pc_eblkno,
 			    pc.pc_eoffset,
 			    &pc.pc_entry,
 			    pcfs.pcfs_entps);
@@ -1853,10 +1992,10 @@ vfs_read_error:
 
 	case N_REGLR:
 	    if (nns) {
-		if ((Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid))
+		if ((Lf->inode = (INODETYPE)nn.nm_vattr.va_nodeid))
 		    Lf->inp_ty = 1;
 	    } else if (ins) {
-		if ((Lf->inode = (unsigned long)i.i_number))
+		if ((Lf->inode = (INODETYPE)i.i_number))
 		    Lf->inp_ty = 1;
 	    }
 	    break;
@@ -1879,7 +2018,7 @@ vfs_read_error:
 		    if (Lf->nma)
 			(void) snpf(&Lf->nma[nl], len - nl, "%s", ubuf);
 		} else {
-		    Lf->inode = (unsigned long)soso.lux_dev.addr.tu_addr.ino;
+		    Lf->inode = (INODETYPE)soso.lux_dev.addr.tu_addr.ino;
 		    Lf->inp_ty = 1;
 		}
 	    }
@@ -1887,14 +2026,14 @@ vfs_read_error:
 
 	    break;
 	case N_TMP:
-	    Lf->inode = (unsigned long)t.tn_attr.va_nodeid;
+	    Lf->inode = (INODETYPE)t.tn_attr.va_nodeid;
 	    Lf->inp_ty = 1;
 	    break;
 
 #if	defined(HASVXFS)
 	case N_VXFS:
 	    if (vx.ino_def) {
-		Lf->inode = vx.ino;
+		Lf->inode = (INODETYPE)vx.ino;
 		Lf->inp_ty = 1;
 	    } else if (type == VCHR)
 		pnl = 1;
@@ -1934,6 +2073,23 @@ vfs_read_error:
 		Lf->sz = (SZOFFTYPE)cn.c_size;
 		Lf->sz_def = 1;
 		break;
+
+#if	solaris>=100000
+	    case N_CTFSADIR:
+	    case N_CTFSBUND:
+	    case N_CTFSCDIR:
+	    case N_CTFSCTL:
+	    case N_CTFSEVT:
+	    case N_CTFSLATE:
+	    case N_CTFSROOT:
+	    case N_CTFSSTAT:
+	    case N_CTFSSYM:
+	    case N_CTFSTDIR:
+	    case N_CTFSTMPL:
+		/* Method of computing CTFS size not known. */
+		break;
+#endif	/* solaris>=100000 */
+
 	    case N_FD:
 		if (v->v_type == VDIR)
 		    Lf->sz = (Unof + 2) * 16;
@@ -2008,6 +2164,14 @@ vfs_read_error:
 		Lf->sz_def = 1;
 		break;
 
+#if	solaris>=100000
+	    case N_PORT:
+		Lf->sz = (SZOFFTYPE)pn.port_curr;
+		Lf->sz_def = 1;
+		break;
+#endif	/* solaris>=100000 */
+
+
 #if	defined(HASPROCFS)
 	    case N_PROC:
 
@@ -2054,7 +2218,12 @@ vfs_read_error:
 /*
  * Record link count.
  */
-	if (Fnlink) {
+
+#if	!defined(HASXOPT)
+	if (Fnlink)
+#endif	/* !defined(HASXOPT) */
+
+	{
 	    switch (Ntype) {
 
 #if	defined(HAS_AFS)
@@ -2073,6 +2242,22 @@ vfs_read_error:
 		break;
 #endif	/* solaris>=20500 */
 
+#if	solaris>=100000
+	    case N_CTFSADIR:
+	    case N_CTFSBUND:
+	    case N_CTFSCDIR:
+	    case N_CTFSCTL:
+	    case N_CTFSEVT:
+	    case N_CTFSLATE:
+	    case N_CTFSROOT:
+	    case N_CTFSSTAT:
+	    case N_CTFSSYM:
+	    case N_CTFSTDIR:
+	    case N_CTFSTMPL:
+		/* Method of computing CTFS link count not known. */
+		break;
+#endif	/* solaris>=100000 */
+
 	case N_FD:
 	    Lf->nlink = (v->v_type == VDIR) ? 2 : 1;
 	    Lf->nlink_def = 1;
@@ -2165,6 +2350,17 @@ vfs_read_error:
 	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
 		Lf->sf |= SELNLINK;
 	}
+
+#if	defined(HASVXFS)
+/*
+ * Record a VxFS file.
+ */
+
+# if	defined(HASVXFSDNLC)
+	Lf->is_vxfs = (Ntype == N_VXFS) ? 1 : 0;
+# endif	/* defined(HASVXFSDNLC) */
+#endif	/* defined(HASVXFS) */
+
 /*
  * Record an NFS selection.
  */
@@ -2231,7 +2427,7 @@ vfs_read_error:
  * use the nm_mountpt's address for name cache lookups.
  */
 	if (nns && devs && (dev == nn.nm_vattr.va_fsid) && (Lf->inp_ty == 1)
-	&&  (Lf->inode == (unsigned long)nn.nm_vattr.va_nodeid))
+	&&  (Lf->inode == (INODETYPE)nn.nm_vattr.va_nodeid))
 	    Lf->na = (KA_T)nn.nm_mountpt;
 # endif	/* defined(HASNCACHE) */
 #endif	/* solaris>=20500 */
@@ -2242,6 +2438,11 @@ vfs_read_error:
 	if (vfs) {
 	    Lf->fsdir = vfs->dir;
 	    Lf->fsdev = vfs->fsname;
+
+#if	defined(HASMNTSTAT)
+	    Lf->mnt_stat = vfs->mnt_stat;
+#endif	/* defined(HASMNTSTAT) */
+
 	    if (!Lf->fsdir && !Lf->fsdev && kvs && fxs) {
 
 	    /*
@@ -2325,6 +2526,16 @@ vfs_read_error:
 	    Lf->rdev_def = rdevs;
 	    break;
 
+#if	solaris>=100000
+	case VPORT:
+	    ty = "PORT";
+	    Lf->dev = dev;
+	    Lf->dev_def = devs;
+	    Lf->rdev = rdev;
+	    Lf->rdev_def = rdevs;
+	    break;
+#endif	/* solaris>=100000 */
+
 #if	solaris>=20600
 	case VPROC:
 
@@ -2407,11 +2618,7 @@ vfs_read_error:
 	    Lf->dev_def = devs;
 	    Lf->rdev = rdev;
 	    Lf->rdev_def = rdevs;
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = (char *)NULL;
 	}
 	if (ty)
@@ -2835,6 +3042,85 @@ read_ncn(na, ca, cn)
 }
 
 
+#if	solaris>=100000
+/*
+ * read_nctfsn(ty, na, ca, cn) - read node's cache node
+ */
+
+static int
+read_nctfsn(ty, na, ca, cn)
+	int ty;				/* node type -- i.e., N_CTFS* */
+	KA_T na;			/* containing node's address */
+	KA_T ca;			/* cache node address */
+	char *cn;			/* CTFS node receiver */
+{
+	char *cp, *nm, tbuf[32];
+	READLEN_T sz;
+
+	switch (ty) {
+	case N_CTFSADIR:
+	    nm = "ADIR";
+	    sz = (READLEN_T)sizeof(ctfs_adirnode_t);
+	    break;
+	case N_CTFSBUND:
+	    nm = "BUND";
+	    sz = (READLEN_T)sizeof(ctfs_bunode_t);
+	    break;
+	case N_CTFSCDIR:
+	    nm = "CDIR";
+	    sz = (READLEN_T)sizeof(ctfs_cdirnode_t);
+	    break;
+	case N_CTFSCTL:
+	    nm = "CTL";
+	    sz = (READLEN_T)sizeof(ctfs_ctlnode_t);
+	    break;
+	case N_CTFSEVT:
+	    nm = "EVT";
+	    sz = (READLEN_T)sizeof(ctfs_evnode_t);
+	    break;
+	case N_CTFSLATE:
+	    nm = "LATE";
+	    sz = (READLEN_T)sizeof(ctfs_latenode_t);
+	    break;
+	case N_CTFSROOT:
+	    nm = "ROOT";
+	    sz = (READLEN_T)sizeof(ctfs_rootnode_t);
+	    break;
+	case N_CTFSSTAT:
+	    nm = "STAT";
+	    sz = (READLEN_T)sizeof(ctfs_ctlnode_t);
+	    break;
+	case N_CTFSSYM:
+	    nm = "SYM";
+	    sz = (READLEN_T)sizeof(ctfs_symnode_t);
+	    break;
+	case N_CTFSTDIR:
+	    nm = "TDIR";
+	    sz = (READLEN_T)sizeof(ctfs_tdirnode_t);
+	    break;
+	case N_CTFSTMPL:
+	    nm = "TMPL";
+	    sz = (READLEN_T)sizeof(ctfs_tmplnode_t);
+	    break;
+	default:
+	    (void) snpf(Namech, Namechl, "unknown CTFS node type: %d", ty);
+	    enter_nm(Namech);
+	    return(1);
+	}
+	if (!ca || kread((KA_T)ca, cn, sz)) {
+	    (void) snpf(Namech, Namechl,
+		"node at %s: can't read CTFS %s node: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		nm,
+		print_kptr(ca, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* solaris>=100000 */
+
+
 /*
  * read_nfn() - read node's fifonode
  */
@@ -3019,7 +3305,7 @@ read_npi(na, v, pids)
 	    if (v->v_type == VDIR) {
 		(void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 		enter_nm(Namech);
-		Lf->inode = PR_ROOTINO;
+		Lf->inode = (INODETYPE)PR_ROOTINO;
 		Lf->inp_ty = 1;
 	    } else {
 		(void) snpf(Namech, Namechl, "/%s/", HASPROCFS);
@@ -3049,13 +3335,13 @@ read_npi(na, v, pids)
 	    return(1);
 	}
 	(void) snpf(Namech, Namechl, "/%s/%d", HASPROCFS, (int)pids->pid_id);
-	Lf->inode = (unsigned long)ptoi(pids->pid_id);
+	Lf->inode = (INODETYPE)ptoi(pids->pid_id);
 	Lf->inp_ty = 1;
 #else	/* solaris>=20600 */
 /*
  * Enter the >= Solaris 2.6 inode number.
  */
-	Lf->inode = (unsigned long)pr.pr_ino;
+	Lf->inode = (INODETYPE)pr.pr_ino;
 	Lf->inp_ty = 1;
 /*
  * Read the >= Solaris 2.6 prnode common structures.
@@ -3291,19 +3577,15 @@ read_npi(na, v, pids)
 	default:
 	    ty = (char *)NULL;
 	}
-	if (!ty) {
-	    if (pr.pr_type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", pr.pr_type);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", pr.pr_type);
-	    (void) snpf(Namech, Namechl,
-		"unknown %s node type", HASPROCFS);
-	} else
+	if (ty)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
+	else
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o",
+		(pr.pr_type & 0xfff));
 /*
  * Record the Solaris 2.6 /proc file system inode number.
  */
-	Lf->inode = (unsigned long)pr.pr_ino;
+	Lf->inode = (INODETYPE)pr.pr_ino;
 	Lf->inp_ty = 1;
 # endif	/* solaris<20600 */
 
@@ -3336,6 +3618,31 @@ read_npn(na, pa, p)
 }
 
 
+#if	solaris>=100000
+/*
+ * read_nprtn() - read node's port node
+ */
+
+static int
+read_nprtn(na, pa, p)
+	KA_T na;			/* containing node's address */
+	KA_T pa;			/* port node address */
+	port_t *p;			/* port node receiver */
+{
+	char tbuf[32];
+
+	if (!pa || kread(pa, (char *)p, sizeof(port_t))) {
+	    (void) snpf(Namech, Namechl, "node at %s: can't read port node: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		print_kptr(pa, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* solaris>=100000 */
+
+
 /*
  * read_nrn() - read node's rnode
  */
@@ -3524,7 +3831,10 @@ savesockmod(so, sop, so_st)
 	struct so_so *sop;		/* previous so_so structure pointer */
 	int *so_st;			/* status of *sop (0 if not loaded) */
 {
+
+#if	solaris<20500
 	dev_t d1, d2, d3;
+#endif	/* solaris<20500 */
 
 #define	luxadr	lux_dev.addr.tu_addr
 #define	luxdev	lux_dev.addr.tu_addr.dev
@@ -3745,18 +4055,46 @@ vop2ty(vp)
 	    return(N_LOFS);
 	else if (Nmvops && Nmvops == (KA_T)vp->v_op)
 	    return(N_NM);
+	else if (Portvops && Portvops == (KA_T)vp->v_op)
+	    return(N_PORT);
 	else if (Prvops && Prvops == (KA_T)vp->v_op)
 	    return(N_PROC);
 	else if (Sam1vops && Sam1vops == (KA_T)vp->v_op)
 	    return(N_SAMFS);
 	else if (Sam2vops && Sam2vops == (KA_T)vp->v_op)
 	    return(N_SAMFS);
+	else if (Sam3vops && Sam3vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
+	else if (Sam4vops && Sam4vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
 	else if ((Sckvops  && Sckvops  == (KA_T)vp->v_op)
 	     ||  (Sncavops && Sncavops == (KA_T)vp->v_op)
 	     ||  (Stpivops && Stpivops == (KA_T)vp->v_op)
 	) {
 	    return(N_SOCK);
 	}
+	else if (Ctfsadir && Ctfsadir == (KA_T)vp->v_op)
+	    return(N_CTFSADIR);
+	else if (Ctfsbund && Ctfsbund == (KA_T)vp->v_op)
+	    return(N_CTFSBUND);
+	else if (Ctfscdir && Ctfscdir == (KA_T)vp->v_op)
+	    return(N_CTFSCDIR);
+	else if (Ctfsctl && Ctfsctl == (KA_T)vp->v_op)
+	    return(N_CTFSCTL);
+	else if (Ctfsevt && Ctfsevt == (KA_T)vp->v_op)
+	    return(N_CTFSEVT);
+	else if (Ctfslate && Ctfslate == (KA_T)vp->v_op)
+	    return(N_CTFSLATE);
+	else if (Ctfsroot && Ctfsroot == (KA_T)vp->v_op)
+	    return(N_CTFSROOT);
+	else if (Ctfsstat && Ctfsstat == (KA_T)vp->v_op)
+	    return(N_CTFSSTAT);
+	else if (Ctfssym && Ctfssym == (KA_T)vp->v_op)
+	    return(N_CTFSSYM);
+	else if (Ctfstdir && Ctfstdir == (KA_T)vp->v_op)
+	    return(N_CTFSTDIR);
+	else if (Ctfstmpl && Ctfstmpl == (KA_T)vp->v_op)
+	    return(N_CTFSTMPL);
 
 # if	defined(HAS_AFS)
 /*
