@@ -1,5 +1,5 @@
 /*
- * machine.h - SunOS (Solaris 1.x and 2.x)  definitions for lsof
+ * machine.h - Solaris definitions for lsof
  */
 
 
@@ -30,13 +30,39 @@
  */
 
 
+/*
+ * $Id: machine.h,v 1.41 2006/03/27 22:57:33 abe Exp $
+ */
+
+
 #if	!defined(LSOF_MACHINE_H)
 #define	LSOF_MACHINE_H	1
 
 
-# if	defined(solaris) && solaris>=20600
+# if	solaris>=90000
 /*
- * <sys/poll.h> must be #include'd for Solaris 2.6 while _KMEMUSER is
+ * <sys/lgrp.h> must be #include'd for Solaris >= 9 before _KMEMUSER or
+ * _KERNEL are defined to avoid a conflict with the use of lgrp_root as
+ * an external symbol in <sys/lgrp.h> and a macro in <sys/lgrp_user.h>.
+ */
+
+#include <sys/lgrp.h>
+# endif	/* solaris>=90000 */
+
+
+# if	solaris>=100000
+/*
+ * Define a dummy aio_req structure for Solaris >= 10, because #include'ing
+ * <sys/aio_req.h> with _KERNEL defined creates too many problems.
+ */
+
+typedef struct aio_req { int dummy; } aio_req_t;
+# endif	/* solaris>=100000 */
+
+
+# if	solaris>=20600
+/*
+ * <sys/poll.h> must be #include'd for Solaris >= 2.6 while _KMEMUSER is
  * defined.  Since <netdb.h> also #include's <sys/poll.h> and <netdb.h>
  * is #include'd from lsof.h, we must perform some early #include magic
  * here to set things up properly.
@@ -56,12 +82,7 @@
 #define	ipa_32	s6_ipaddr.ipa_32
 # endif	/* solaris>=80000 */
 
-# endif	/* defined(solaris) && solaris>=20600 */
-
-
-/*
- * $Id: machine.h,v 1.20 99/10/22 08:23:52 abe Exp $
- */
+# endif	/* solaris>=20600 */
 
 
 /*
@@ -69,9 +90,21 @@
  * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
  */
 
-#if	!defined(solaris) || (defined(solaris) && solaris>=20501)
+# if	solaris>=20501
 #define	CAN_USE_CLNT_CREATE	1
-#endif	/* !defined(solaris) || (defined(solaris) && solaris>=20501) */
+# endif	/* solaris>=20501 */
+
+
+/*
+ * DEVDEV_PATH defines the path to the directory that contains device
+ * nodes.
+ */
+
+# if	solaris<100000
+#define	DEVDEV_PATH	"/dev"
+# else	/* solaris>=100000 */
+#define	DEVDEV_PATH	"/devices"
+# endif	/* solaris<100000 */
 
 
 /*
@@ -79,9 +112,9 @@
  * getdtablesize() to obtain the maximum file descriptor number plus one.
  */
 
-#if	defined(solaris) && solaris<20500
+# if	solaris<20500
 #define	GET_MAX_FD	get_max_fd
-#endif	/* defined(solaris) && solaris<20500 */
+# endif	/* solaris<20500 */
 
 
 /*
@@ -161,10 +194,22 @@
  * FSV_DEFAULT defines the default set of file structure values to list.
  * It defaults to zero (0), but may be made up of a combination of the
  * FSV_* symbols from lsof.h.
+ *
+ * If any file structure value is unavailable, its use may be suppressed
+ * with any of the following definitions:
+ *
+ *   HASNOFSADDR  -- has no file structure address
+ *   HASNOFSFLAGS -- has no file structure flags
+ *   HASNOFSCOUNT -- has no file structure count
+ *   HASNOFSNADDR -- has no file structure node address
  */
 
 #define	HASFSTRUCT	1
 /* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+/* #define	HASNOFSADDR	1	has no file structure address */
+/* #define	HASNOFSFLAGS	1	has no file structure flags */
+/* #define	HASNOFSCOUNT	1	has no file structure count */
+/* #define	HASNOFSNADDR	1	has no file structure node address */
 
 
 /*
@@ -186,9 +231,7 @@
  * use readinode() from node.c.
  */
 
-#if	!defined(solaris)
-#define	HASINODE	1
-#endif	/* !defined(solaris) */
+/* #define	HASINODE	1	*/
 
 
 /*
@@ -217,15 +260,45 @@
 
 /*
  * HASLFILEADD is defined for those dialects that need additional elements
- * in struct lfile.  The HASLFILEADD definition is a macro that defines
- * them.  If any of the additional elements need to be preset in the
- * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
- * to do that.  The HASXOPT definition may be used to select the conditions
- * under which private lfile elements are used.
+ * in struct lfile.  The HASLFILEADD definition is a macro that defines them.
+ *
+ * If any additional elements need to be preset in the alloc_lfile() function
+ * of proc.c, the SETLFILEADD macro may be defined to do that.
+ *
+ * If any additional elements need to be cleared in alloc_lfile() or in the
+ * free_proc() function of proc.c, the CLRLFILEADD macro may be defined to
+ * do that.  Note that CLRLFILEADD takes one argument, the pointer to the
+ * lfile struct.  The CLRLFILEADD macro is expected to expand to statements
+ * that are complete -- i.e., have terminating semi-colons -- so the macro is
+ * called without a terminating semicolon by proc.c.
+ *
+ * The HASXOPT definition may be used to select the conditions under which
+ * private lfile elements are used.
  */
 
-/* #define HASLFILEADD int ... */
-/* #define SETLFILEADD Lf->... */
+# if	solaris>=10000 && defined(HAS_V_PATH)
+#define	HASLFILEADD	KA_T V_path;
+#define	CLRLFILEADD(lf)	(lf)->V_path = (KA_T)NULL;
+#define SETLFILEADD Lf->V_path = (KA_T)NULL;
+# endif	/* solaris>=10000 && defined(HAS_V_PATH) */
+
+
+/*
+ * HASMNTSTAT indicates the dialect supports the mount stat(2) result option
+ * in its l_vfs and mounts structures.
+ */
+
+# if	solaris>=10000 && defined(HAS_V_PATH)
+#define	HASMNTSTAT	1
+# endif	/* solaris>=10000 && defined(HAS_V_PATH) */
+
+
+/*
+ * HASMNTSUP is defined for those dialects that support the mount supplement
+ * option.
+ */
+
+/* #define	HASMNTSUP	1	*/
 
 
 /*
@@ -238,14 +311,23 @@
 
 /*
  * HASNCACHE is defined for those dialects that have a kernel name cache
- * that lsof can search.
+ * that lsof can search.  A value of 1 directs printname() to prefix the
+ * cache value with the file system directory name; 2, avoid the prefix.
  *
- * HASNCAPID is defined for those dialects with a searchable kernel name
- * cache whose cache and vnodes are linked by a capability ID.
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
+# if	solaris>=10000 && defined(HAS_V_PATH)
+/* #define	HASNCACHE	1	*/
+#else	/* solaris<10 || !defined(HAS_V_PATH) */
 #define	HASNCACHE	1
-/* #define	HASNCAPID	1	*/
+# endif	/* solaris>=10000 && defined(HAS_V_PATH) */
+
+#define	NCACHELDPFX	open_kvm();	/* do before calling ncache_load() */
+
+/* #define	NCACHELDSFX	??? */
 
 
 /*
@@ -297,12 +379,8 @@
  * called from print_file().
  */
 
-#if	defined(solaris)
 #define	HASPRINTDEV	print_dev
-#else	/* !defined(solaris) */
-#define	HASPRINTINO	print_ino
-#endif	/* defined(solaris) */
-
+/* #define	HASPRINTINO	print_ino?	*/
 /* #define	HASPRINTNM	print_nm?	*/
 /* #define	HASPRINTOFF	print_off?	*/
 /* #define	HASPRINTSZ	print_sz?	*/
@@ -331,15 +409,38 @@
  * returns non-zero if it prints a name to stdout.
  */
 
+# if	solaris>=10000 && defined(HAS_V_PATH)
+#define	HASPRIVNMCACHE	print_v_path
+# else	/* solaris<10 || !defined(HAS_V_PATH) */
+#  if	defined(HASVXFSRNL)
+#define	HASPRIVNMCACHE	print_vxfs_rnl_path
+#  else	/* !defined(HASVXFSRNL) */
 /* #define	HASPRIVNMCACHE	<function name>	*/
+#  endif	/* defined(HASVXFSRNL) */
+# endif	/* solaris>=10000 && defined(HAS_V_PATH) */
+
+
+/*
+ * HASPRIVPRIPP is defined for dialects that have a private function for
+ * printing IP protocol names.  When HASPRIVPRIPP isn't defined, the
+ * IP protocol name printing function defaults to printiprto().
+ */
+
+/* #define	HASPRIVPRIPP	1	*/
 
 
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
  *
- * HASFSTYPE is defined for those systems that have a file system type
- * string, st_fstype, in the stat() buffer.
+ * HASFSTYPE is defined as 1 for those systems that have a file system type
+ * string, st_fstype, in the stat() buffer; 2, for those systems that have a
+ * file system type integer in the stat() buffer, named MOUNTS_STAT_FSTYPE;
+ * 0, for systems whose stat(2) structure has no file system type member.  The
+ * additional symbols MOUNTS_FSTYPE, RMNT_FSTYPE, and RMNT_STAT_FSTYPE may be
+ * defined in dlsof.h to direct how the readmnt() function in lib/rmnt.c
+ * preserves these stat(2) and getmntent(3) buffer values in the local mounts
+ * structure.
  *
  * The defined value is the string that names the file system type.
  *
@@ -352,11 +453,9 @@
  * by inode number.
  */
 
-# if	defined(solaris)
 #define	HASPROCFS	"proc"
 #define	HASFSTYPE	1
 #define	HASPINODEN	1
-# endif	/* defined(solaris) */
 
 
 /*
@@ -376,6 +475,36 @@
 /* #define	HASSECURITY	1	*/
 
 
+/*
+ * If HASSECURITY is defined, define HASNOSOCKSECURITY to allow users
+ * restricted by HASSECURITY to list any open socket files, provide their
+ * listing is selected by the "-i" option.
+ */
+
+/* #define	HASNOSOCKSECURITY	1	*/
+
+
+/*
+ * HASSETLOCALE is defined for those dialects that have <locale.h> and
+ * setlocale().
+ *
+ * If the dialect also has wide character support for language locales,
+ * HASWIDECHAR activates lsof's wide character support and WIDECHARINCL
+ * defines the header file (if any) that must be #include'd to use the
+ * mblen() and mbtowc() functions.
+ *
+ * If a special definition is required (e.g., for Solaris) before #include'ing
+ * <ctype.h>, do that here.
+ */
+
+#define	HASSETLOCALE	1
+#define	HASWIDECHAR	1
+#define	WIDECHARINCL	<wchar.h>
+#define __XPG4_CHAR_CLASS__
+#include	<ctype.h>
+#undef __XPG4_CHAR_CLASS__
+
+
 /*
  * HASSNODE is defined for those dialects that have snodes.
  */
@@ -383,6 +512,37 @@
 #define	HASSNODE	1
 
 
+/*
+ * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
+ * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
+ * options.
+ */
+
+# if	solaris>=20600
+#define	HASSOOPT	1	/* has socket option information */
+/* #define	HASSOSTATE	1	has socket state information */
+#define	HASTCPOPT	1	/* has TCP options or flags */
+# endif	/* solaris>=20600 */
+
+
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -395,9 +555,9 @@
  * TCP/TPI Recv-Q and Send-Q values produced by netstat.
  */
 
-#if	!defined(solaris) || solaris==20300 || solaris>=20500
+# if	solaris==20300 || solaris>=20500
 #define	HASTCPTPIQ	1
-#endif	/* !defined(solaris) || solaris==20300 || solaris>=20500 */
+# endif	/* solaris==20300 || solaris>=20500 */
 
 
 /*
@@ -405,9 +565,9 @@
  * TCP/TPI send and receive window sizes produced by netstat.
  */
 
-#if	defined(solaris) && (solaris==20300 || solaris>=20500)
+# if	solaris==20300 || solaris>=20500
 #define	HASTCPTPIW	1
-#endif	/* defined(solaris) && (solaris==20300 || solaris>=20500) */
+# endif	/* solaris==20300 || solaris>=20500 */
 
 
 /*
@@ -432,8 +592,26 @@
  * option's default binary value -- 0 or 1.
  */
 
-/* #define	HASXOPT		"help text for X option" */
-/* #define	HASXOPT_VALUE	1 */
+# if	solaris>=10000 && defined(HAS_V_PATH)
+#define	HASXOPT		"report deleted paths"
+#define	HASXOPT_VALUE	0
+# endif	/* solaris>=10000 && defined(HAS_V_PATH) */
+
+
+/*
+ * INODETYPE and INODEPSPEC define the internal node number type and its
+ * printf specification modifier.  These need not be defined and lsof.h
+ * can be allowed to define defaults.
+ *
+ * These are defined here, because they must be used in dlsof.h.
+ */
+
+# if	solaris>=20501
+#define	INODETYPE	unsigned long long
+					/* inode number internal storage type */
+#define	INODEPSPEC	"ll"		/* INODETYPE printf specification
+					 * modifier */
+# endif	/* solaris>=20501 */
 
 
 /*
@@ -458,19 +636,26 @@
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 /* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
 #define	USE_LIB_LKUPDEV				1	/* lkud.c */
-/* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
+/* #define	USE_LIB_PRINTDEVNAME		1	   pdvn.c */
 /* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
-
-#if	!defined(solaris)
-#define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
-#endif	/* !defined(solaris) */
-
+/* #define	USE_LIB_PRINT_TCPTPI		1	   ptti.c */
 /* #define	USE_LIB_READDEV			1	   rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
+/* #define	USE_LIB_REGEX			1	   regex.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
+
+# if	solaris<90000
 #define	USE_LIB_RNCH				1	/* rnch.c */
+# endif	/* solaris<90000 */
+
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
 
+# if	solaris<20600
+#define	USE_LIB_SNPF				1	/* snpf.c */
+# else	/* solaris>=20600 */
+#define	snpf	snprintf	   /* use the system's snprintf() */
+# endif	/* solaris<20600 */
+
 
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
@@ -502,10 +687,6 @@
  * zeromem is a macro that uses bzero or memset.
  */
 
-# if	defined(solaris)
 #define	zeromem(a, l)		memset((void *)a, 0, l)
-# else	/* !defined(solaris) */
-#define	zeromem(a, l)		memset((char *)a, 0, l)
-# endif	/* defined(solaris) */
 
 #endif	/* !defined(LSOF_MACHINE_H) */
