@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rnch.c,v 1.6 98/03/24 10:40:51 abe Exp $";
+static char *rcsid = "$Id: rnch.c,v 1.10 2005/08/08 19:41:55 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -89,6 +89,11 @@ static char *rcsid = "$Id: rnch.c,v 1.6 98/03/24 10:40:51 abe Exp $";
  *
  *				Deafult: namlen
  *
+ *	Define NCACHE_NEGVN	as the name of the name list element
+ *				whose value is a vnode address to
+ *				ignore when loading the kernel name
+ *				cache.
+ *
  *	Define NCACHE_NODEID	as the name of the element in the
  *				ncache structure that contains the
  *				vnode's capability ID.
@@ -147,6 +152,11 @@ static struct l_nch **Nchash = (struct l_nch **)NULL;
 					/* Ncache hash pointers */
 static int Ncfirst = 1;			/* first-call status */
 
+# if 	defined(NCACHE_NEGVN)
+static KA_T NegVN = (KA_T)NULL;		/* negative vnode address */
+static int NegVNSt = 0;			/* NegVN status: 0 = not loaded */
+# endif	/* defined(NCACHE_NEGVN) */
+
 # if	defined(NCACHE_NODEID)
 _PROTOTYPE(static struct l_nch *ncache_addr,(unsigned long i, KA_T v));
 #define ncachehash(i,v)		Nchash+(((((int)(v)>>2)+((int)(i)))*31415)&Mch)
@@ -350,7 +360,7 @@ ncache_load()
 		if (!Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: can't read name cache size: %s\n",
-			Pn, print_kptr(v, (char *)NULL));
+			Pn, print_kptr(v, (char *)NULL, 0));
 		iNc = Nc = 0;
 		return;
 	    }
@@ -368,6 +378,19 @@ ncache_load()
 		}
 		iNc = Nc = DEFNCACHESZ;
 	    }
+
+# if	defined(NCACHE_NEGVN)
+	/*
+	 * Get negative vnode address.
+	 */
+	    if (!NegVNSt) {
+		if (get_Nl_value(NCACHE_NEGVN, (struct drive_Nl *)NULL, &NegVN)
+		< 0)
+		    NegVN = (KA_T)NULL;
+		NegVNSt = 1;
+	    }
+# endif	/* defined(NCACHE_NEGVN) */
+
 	/*
 	 * Establish kernel cache address.
 	 */
@@ -390,7 +413,7 @@ ncache_load()
 		if (!Fwarn)
 		    (void) fprintf(stderr,
 			"%s: WARNING: can't read name cache ptr: %s\n",
-			Pn, print_kptr(v, (char *)NULL));
+			Pn, print_kptr(v, (char *)NULL, 0));
 		iNc = Nc = 0;
 		return;
 	    }
@@ -414,7 +437,7 @@ ncache_load()
 	 * Allocate space for the local cache.
 	 */
 	    len = Nc * sizeof(struct l_nch);
-	    if (!(Ncache = (struct l_nch *)malloc((MALLOC_S)len))) {
+	    if (!(Ncache = (struct l_nch *)calloc(Nc, sizeof(struct l_nch)))) {
 
 no_local_space:
 
@@ -463,7 +486,7 @@ no_local_space:
 	    if (!Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: can't read kernel's name cache: %s\n",
-		    Pn, print_kptr(kp, (char *)NULL));
+		    Pn, print_kptr(kp, (char *)NULL, 0));
 	    Nc = 0;
 	    return;
 	}
@@ -491,6 +514,11 @@ no_local_space:
 	    if (!kc->NCACHE_VP || (len = kc->NCACHE_NAMLEN) < 1)
 		continue;
 
+# if	defined(NCACHE_NEGVN)
+	    if (NegVN && ((KA_T)kc->NCACHE_VP == NegVN))
+		continue;
+# endif	/* defined(NCACHE_NEGVN) */
+
 # if	defined(HASDNLCPTR)
 	/*
 	 * Read name from kernel to a temporary buffer.
@@ -671,6 +699,13 @@ no_local_space:
 	    if (!lc->dp)
 		continue;
 
+# if	defined(NCACHE_NEGVN)
+	     if (NegVN && (lc->dp == NegVN)) {
+		lc->pa = (struct l_nch *)NULL;
+		continue;
+	     }
+# endif	/* defined(NCACHE_NEGVN) */
+
 # if	defined(NCACHE_NODEID)
 	    lc->pa = ncache_addr(lc->did, lc->dp);
 # else	/* !defined(NCACHE_NODEID) */
@@ -705,7 +740,7 @@ ncache_lookup(buf, blen, fp)
  * file system mount point, return an empty path reply.  That tells the
  * caller to print the file system mount point name only.
  */
-	if (Lf->inp_ty == 1 && Lf->fs_ino && Lf->inode == Lf->fs_ino)
+	if ((Lf->inp_ty == 1) && Lf->fs_ino && (Lf->inode == Lf->fs_ino))
 	    return(cp);
 # endif	/* defined(HASFSINO) */
 
@@ -732,7 +767,7 @@ ncache_lookup(buf, blen, fp)
 		if (!mtp->dir || !mtp->inode)
 		    continue;
 		if (Lf->dev == mtp->dev
-		&&  (unsigned long)mtp->inode == Lf->inode
+		&&  mtp->inode == Lf->inode
 		&&  strcmp(mtp->dir, Lf->fsdir) == 0)
 		    return(cp);
 	    }
