@@ -38,7 +38,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rnmh.c,v 1.4 98/03/24 10:41:15 abe Exp $";
+static char *rcsid = "$Id: rnmh.c,v 1.11 2005/08/08 19:42:23 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -74,11 +74,14 @@ static char *rcsid = "$Id: rnmh.c,v 1.4 98/03/24 10:41:15 abe Exp $";
  *	Define the names of these elements of struct NCACHE:
  *
  *		#define NCACHE_NM	<name>
- *		#define NCACHE_NMLEN	<name length
  *		#define NCACHE_NXT	<link to next entry>
  *		#define NCACHE_NODEADDR	<node address>
  *		#define NCACHE_PARADDR	<parent node address>
  *
+ *	Optionally define:
+ *
+ *		#define NCACHE_NMLEN	<name length>
+ *
  *	Optionally define *both*:
  *
  *		#define	NCACHE_NODEID	<node capability ID>
@@ -90,9 +93,23 @@ static char *rcsid = "$Id: rnmh.c,v 1.4 98/03/24 10:41:15 abe Exp $";
  *
  *		_PROTOTYPE(static void ncache_load,(void));
  *
+ *	Define NCACHE_VROOT to be the value of the flag that signifies that
+ *	the vnode is the root of its file system.
+ *
+ *		E.g., for BSDI >= 5:
+ *
+ *			#define NCACHE_VROOT	VV_ROOT
+ *
+ *	If not defined, NCACHE_VROOT is defined as "VROOT".
+ *
+ *	Define VNODE_VFLAG if the vnode's flag member's name isn't v_flag.
+ *
  * Note: if NCHNAMLEN is defined, the name is assumed to be in
  * NCACHE_NM[NCHNAMLEN]; if it isn't defined, the name is assumed to be in an
  * extension that begins at NCACHE_NM[0].
+ *
+ * Note: if NCACHE_NMLEN is not defined, then NCACHE_NM must be a pointer to
+ * a kernel allocated, NUL-terminated, string buffer.
  */
 
 
@@ -105,6 +122,23 @@ static char *rcsid = "$Id: rnmh.c,v 1.4 98/03/24 10:41:15 abe Exp $";
 # endif	/* !defined(NCACHE_NC_CAST) */
 
 
+/*
+ * Flags
+ */
+
+# if	!defined(NCACHE_NMLEN)
+#undef	NCHNAMLEN
+# endif	/* !defined(NCACHE_NMLEN) */
+
+# if	!defined(NCACHE_VROOT)
+#define	NCACHE_VROOT	VROOT		/* vnode is root of its file system */
+# endif	/* !defined(NCACHE_VROOT) */
+
+# if	!defined(VNODE_VFLAG)
+#define	VNODE_VFLAG	v_flag
+# endif	/* !defined(VNODE_VFLAG) */
+
+
 /*
  * Local static values
  */
@@ -216,8 +250,8 @@ ncache_isroot(na, cp)
 		return(1);
 	}
 /*
- * Read the vnode and see if it's a VDIR node with the VROOT flag set.  If
- * it is, then the path is complete.
+ * Read the vnode and see if it's a VDIR node with the NCACHE_VROOT flag set.
+ * If it is, then the path is complete.
  *
  * If it isn't, and if the file has an inode number, search the mount table
  * and see if the file system's inode number is known.  If it is, form the
@@ -225,7 +259,7 @@ ncache_isroot(na, cp)
  * the one we have for this file.  If it does, then the path is complete.
  */
 	if (kread((KA_T)na, (char *)&v, sizeof(v))
-	||  v.v_type != VDIR || !(v.v_flag & VROOT)) {
+	||  v.v_type != VDIR || !(v.VNODE_VFLAG & NCACHE_VROOT)) {
 
 	/*
 	 * The vnode tests failed.  Try the inode tests.
@@ -295,6 +329,21 @@ ncache_load()
 	char tbuf[32];
 	KA_T v;
 
+# if	!defined(NCHNAMLEN)
+	int cin = sizeof(c.NCACHE_NM);
+	KA_T nmo = (KA_T)offsetof(struct NCACHE, NCACHE_NM);
+# endif	/* !defined(NCHNAMLEN) */
+
+# if	!defined(NCACHE_NMLEN)
+	char nbf[MAXPATHLEN + 1];
+	int nbfl = (int)(sizeof(nbf) - 1);
+	KA_T nk;
+	char *np;
+	int rl;
+
+	nbf[nbfl] = '\0';
+# endif	/* !defined(NCACHE_NMLEN) */
+
 	if (!Fncache)
 	    return;
 /*
@@ -319,7 +368,7 @@ ncache_load()
 	    if (!Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: can't read name cache hash size: %s\n",
-		    Pn, print_kptr(v, (char *)NULL));
+		    Pn, print_kptr(v, (char *)NULL, 0));
 	    return;
 	}
 	if (khsz < 1) {
@@ -340,9 +389,10 @@ ncache_load()
 	||  !ka)
 	{
 	    if (!Fwarn)
-	      (void) fprintf(stderr,
-		  "%s: WARNING: unusable name cache hash pointer: (%s)=%s\n",
-	          Pn, print_kptr(v, tbuf), print_kptr(ka, (char *)NULL));
+		(void) fprintf(stderr,
+		    "%s: WARNING: unusable name cache hash pointer: (%s)=%s\n",
+	            Pn, print_kptr(v, tbuf, sizeof(tbuf)),
+		    print_kptr(ka, (char *)NULL, 0));
 	    return;
 	}
 /*
@@ -365,7 +415,7 @@ ncache_load()
 	if (kread((KA_T)ka, (char *)khp, len)) {
 	    (void) fprintf(stderr,
 		"%s: can't read name cache hash pointers from: %s\n",
-		Pn, print_kptr(ka, (char *)NULL));
+		Pn, print_kptr(ka, (char *)NULL, 0));
 	    return;
 	}
 /*
@@ -388,9 +438,54 @@ ncache_load()
 		if (kread(ka, (char *)&c, sizeof(c)))
 		    break;
 		knx = (KA_T)c.NCACHE_NXT;
-		if (!c.NCACHE_NODEADDR || !(len = c.NCACHE_NMLEN))
+		if (!c.NCACHE_NODEADDR)
 		    continue;
 
+# if	defined(NCACHE_NMLEN)
+		if (!(len = c.NCACHE_NMLEN))
+		    continue;
+# else	/* !defined(NCACHE_NMLEN) */
+	    /*
+	     * If it's possible to read the first four characters of the name,
+	     * do so and check for "." and "..".
+	     */
+		if (!c.NCACHE_NM
+		||  kread((KA_T)c.NCACHE_NM, nbf, 4))
+		    continue;
+		if (nbf[0] == '.') {
+		    if (!nbf[1]
+		    ||  ((nbf[1] == '.') && !nbf[2]))
+			continue;
+		}
+	   /*
+	    * Read the rest of the name, 32 characters at a time, until a NUL
+	    * character has been read or nbfl characters have been read.
+	    */
+		nbf[4] = '\0';
+		if ((len = (int)strlen(nbf)) < 4) {
+		    if (!len)
+			continue;
+		} else {
+		    for (np = &nbf[4]; len < nbfl; np += rl) {
+			if ((rl = nbfl - len) > 32) {
+			    rl = 32;
+			    nbf[len + rl] = '\0';
+			}
+			nk = (KA_T)((char *)c.NCACHE_NM + len);
+			if (kread(nk, np, rl)) {
+			    rl = -1;
+			    break;
+			}
+			rl = (int)strlen(np);
+			len += rl;
+			if (rl < 32)
+			    break;
+		    }
+		    if (rl < 0)
+			continue;
+		}
+# endif	/* defined(NCACHE_NMLEN) */
+
 	    /*
 	     * Allocate a cache entry long enough to contain the name and
 	     * move the name to it.
@@ -425,15 +520,42 @@ ncache_load()
 # if	defined(NCHNAMLEN)
 		(void) strncpy(lc->nm, c.NCACHE_NM, len);
 # else	/* !defined(NCHNAMLEN) */
-		if (kread(ka + (KA_T)(offsetof(struct NCACHE, NCACHE_NM)),
-		          lc->nm, len))
-		    continue;
-		if (len < 3 && lc->nm[0] == '.') {
-		    if (len == 1 || (len == 2 && lc->nm[1] == '.'))
-			continue;
-		}
-# endif	/* defined(NCHNAMLEN) */
+#  if	defined(NCACHE_NMLEN)
+		if ((len < 3) && (cin > 1)) {
 
+		/*
+		 * If this is a one or two character name, and if NCACHE_NM[]
+		 * in c has room for at least two characters, check for "."
+		 * and ".." first, ignoring this entry if the name is either.
+		 */
+		    if (len < 3 && c.NCACHE_NM[0] == '.') {
+			if (len == 1 || (len == 2 && c.NCACHE_NM[1] == '.'))
+			    continue;
+		    }
+		}
+		if (len > cin) {
+
+		/*
+		 * If not all (possibly not any, depending on the value in
+		 * cin) of the name has yet been read to lc->nm[], read it
+		 * or the rest of it.  If it wasn't possible before to check
+		 * for "." or "..", do that. too.
+		 */
+		    if (cin > 0)
+			(void) strncpy(lc->nm, c.NCACHE_NM, cin);
+		    if (kread(ka + (KA_T)(nmo + cin), &lc->nm[cin], len - cin))
+			continue;
+		    if ((cin < 2) && (len < 3) && (lc->nm[0] == '.')) {
+			if (len == 1 || (len == 2 && lc->nm[1] == '.'))
+			    continue;
+		    }
+		} else
+		    (void) strncpy(lc->nm, c.NCACHE_NM, len);
+#  else	/* !defined(NCACHE_NMLEN) */
+		(void) strncpy(lc->nm, nbf, len);
+#  endif	/* defined(NCACHE_NMLEN) */
+
+# endif	/* defined(NCHNAMLEN) */
 		lc->nm[len] = '\0';
 	    /*
 	     * Complete the new local cache entry and link it to the previous
@@ -547,7 +669,7 @@ ncache_lookup(buf, blen, fp)
  * file system mount point, return an empty path reply.  That tells the
  * caller to print the file system mount point name only.
  */
-	if (Lf->inp_ty == 1 && Lf->fs_ino && Lf->inode == Lf->fs_ino)
+	if ((Lf->inp_ty == 1) && Lf->fs_ino && (Lf->inode == Lf->fs_ino))
 	    return(cp);
 # endif	/* defined(HASFSINO) */
 
@@ -573,8 +695,8 @@ ncache_lookup(buf, blen, fp)
 		if (!mtp->dir || !mtp->inode)
 		    continue;
 		if (Lf->dev == mtp->dev
-		&&  (unsigned long)mtp->inode == Lf->inode
-		&&  strcmp(mtp->dir, Lf->fsdir) == 0)
+		&&  mtp->inode == Lf->inode
+		&&  (strcmp(mtp->dir, Lf->fsdir) == 0))
 		    return(cp);
 	    }
 	    return((char *)NULL);
