@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.34 99/11/28 06:34:37 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.47 2006/03/27 23:04:25 abe Exp $";
 #endif
 
 
@@ -91,12 +91,15 @@ _PROTOTYPE(static void update_portmap,(struct porttab *pt, char *pn));
  */
 
 char *
-endnm()
+endnm(sz)
+	size_t *sz;			/* returned remaining size */
 {
-	char *s;
+	register char *s;
+	register size_t tsz;
 
-	for (s = Namech; *s; s++)
+	for (s = Namech, tsz = Namechl; *s; s++, tsz--)
 		;
+	*sz = tsz;
 	return(s);
 }
 
@@ -226,7 +229,8 @@ fill_portmap()
 		    cp = r->r_name;
 	    }
 	    if (!cp) {
-		(void) sprintf(buf, "%lu", (unsigned long)p->pml_map.pm_prog);
+		(void) snpf(buf, sizeof(buf), "%lu",
+			    (unsigned long)p->pml_map.pm_prog);
 		cp = buf;
 	    }
 	    if (!strlen(cp))
@@ -393,17 +397,21 @@ gethostnm(ia, af)
 	     * them in brackets.
 	     */
 		hbuf[0] = '[';
-		if (!inet_ntop(af, ia, hbuf + 1, sizeof(hbuf) - 3))
-		    (void) strcpy(&hbuf[1], "can't format IPv6 address]");
-		else
-		    (void) strcat(hbuf, "]");
+		if (!inet_ntop(af, ia, hbuf + 1, sizeof(hbuf) - 3)) {
+		    (void) snpf(&hbuf[1], (sizeof(hbuf) - 1),
+			"can't format IPv6 address]");
+		} else {
+		    len = strlen(hbuf);
+		    (void) snpf(&hbuf[len], sizeof(hbuf) - len, "]");
+		}
 	    } else
 #endif	/* defined(HASIPv6) */
 
 	    if (af == AF_INET)
-		(void) sprintf(hbuf, "%u.%u.%u.%u", ia[0], ia[1], ia[2], ia[3]);
+		(void) snpf(hbuf, sizeof(hbuf), "%u.%u.%u.%u", ia[0], ia[1],
+			    ia[2], ia[3]);
 	    else
-		(void) sprintf(hbuf, "(unknown AF value: %d)", af);
+		(void) snpf(hbuf, sizeof(hbuf), "(unknown AF value: %d)", af);
 	    hn = hbuf;
 	} else
 	    hn = (char *)he->h_name;
@@ -495,7 +503,7 @@ lkup_port(p, pr, src)
 		if (!pt->ss) {
 		    pn = Fport ? lkup_svcnam(h, p, pr, 0) : (char *)NULL;
 		    if (!pn) {
-			(void) sprintf(pb, "%d", p);
+			(void) snpf(pb, sizeof(pb), "%d", p);
 			pn = pb;
 		    }
 		    (void) update_portmap(pt, pn);
@@ -518,7 +526,7 @@ lkup_port(p, pr, src)
 	pn = Fport ? lkup_svcnam(h, p, pr, 1) : (char *)NULL;
 	if (!pn || !strlen(pn)) {
 	    if (p) {
-		(void) sprintf(pb, "%d", p);
+		(void) snpf(pb, sizeof(pb), "%d", p);
 		return(pb);
 	    } else
 		return("*");
@@ -591,9 +599,9 @@ lkup_svcnam(h, p, pr, ss)
 		}
 	    }
 /*
- * If fill_port() has been called, there is no service name.
+ * If fill_porttab() has been called, there is no service name.
  *
- * Do PORTTABTHRES getservbport() calls, remembering the faulures, so they
+ * Do PORTTABTHRES getservbport() calls, remembering the failures, so they
  * won't be repeated.
  *
  * After PORTABTHRESH getservbyport() calls, call fill_porttab() once,
@@ -602,7 +610,7 @@ lkup_svcnam(h, p, pr, ss)
 		break;
 	    if (gsbp < PORTTABTHRESH) {
 		for (i = 0; i < fln; i++) {
-		    if(fl[i] == p)
+		    if (fl[i] == p)
 			return((char *)NULL);
 		}
 		gsbp++;
@@ -629,7 +637,8 @@ print_file()
 {
 	char buf[128];
 	char *cp = (char *)NULL;
-	int len;
+	dev_t dev;
+	int devs, len;
 
 	if (PrPass && !Hdr) {
 
@@ -637,16 +646,21 @@ print_file()
 	 * Print the header line if this is the second pass and the
 	 * header hasn't already been printed.
 	 */
-	    (void) printf("%-*.*s %*s", CmdColW, CmdColW, CMDTTL,
-		PidColW, PIDTTL);
+	    (void) printf("%-*.*s %*s", CmdColW, CmdColW, CMDTTL, PidColW,
+		PIDTTL);
+
+#if	defined(HASZONES)
+	    if (Fzone)
+	        (void) printf(" %-*s", ZoneColW, ZONETTL);
+#endif	/* defined(HASZONES) */
 
 #if	defined(HASPPID)
 	    if (Fppid)
 	 	(void) printf(" %*s", PpidColW, PPIDTTL);
 #endif	/* defined(HASPPID) */
 
-	    if (Fpgrp)
-		(void) printf(" %*s", PgrpColW, PGRPTTL);
+	    if (Fpgid)
+		(void) printf(" %*s", PgidColW, PGIDTTL);
 	    (void) printf(" %*s %*s   %*s",
 		UserColW, USERTTL,
 		FdColW - 2, FDTTL,
@@ -654,14 +668,27 @@ print_file()
 
 #if	defined(HASFSTRUCT)
 	    if (Fsv) {
+
+# if	!defined(HASNOFSADDR)
 		if (Fsv & FSV_FA)
 		    (void) printf(" %*s", FsColW, FSTTL);
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSCOUNT)
 		if (Fsv & FSV_CT)
 		    (void) printf(" %*s", FcColW, FCTTL);
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSFLAGS)
 		if (Fsv & FSV_FG)
 		    (void) printf(" %*s", FgColW, FGTTL);
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 		if (Fsv & FSV_NI)
 		    (void) printf(" %*s", NiColW, NiTtl);
+# endif	/* !defined(HASNOFSNADDR) */
+
 	    }
 #endif	/* defined(HASFSTRUCT) */
 
@@ -682,8 +709,9 @@ print_file()
  */
 	cp = (Lp->cmd && *Lp->cmd != '\0') ? Lp->cmd : "(unknown)";
 	if (!PrPass) {
-	    if ((len = safestrlen(cp, 2)) > CMDL)
-		len = CMDL;
+	    len = safestrlen(cp, 2);
+	    if (CmdLim && (len > CmdLim))
+		len = CmdLim;
 	    if (len > CmdColW)
 		CmdColW = len;
 	} else
@@ -692,12 +720,27 @@ print_file()
  * Size or print the process ID.
  */
 	if (!PrPass) {
-	    (void) sprintf(buf, "%d", Lp->pid);
+	    (void) snpf(buf, sizeof(buf), "%d", Lp->pid);
 	    if ((len = strlen(buf)) > PidColW)
 		PidColW = len;
 	} else
 	    (void) printf(" %*d", PidColW, Lp->pid);
 
+#if	defined(HASZONES)
+/*
+ * Size or print the zone.
+ */
+	if (Fzone) {
+	    if (!PrPass) {
+		if (Lp->zn) {
+		    if ((len = strlen(Lp->zn)) > ZoneColW)
+			ZoneColW = len;
+		}
+	    } else
+		(void) printf(" %-*s", ZoneColW, Lp->zn ? Lp->zn : "");
+	}
+#endif	/* defined(HASZONES) */
+
 #if	defined(HASPPID)
 	if (Fppid) {
 
@@ -705,7 +748,7 @@ print_file()
 	 * Size or print the parent process ID.
 	 */
 	    if (!PrPass) {
-		(void) sprintf(buf, "%d", Lp->ppid);
+		(void) snpf(buf, sizeof(buf), "%d", Lp->ppid);
 		if ((len = strlen(buf)) > PpidColW)
 		    PpidColW = len;
 	    } else
@@ -713,17 +756,17 @@ print_file()
 	}
 #endif	/* defined(HASPPID) */
 
-	if (Fpgrp) {
+	if (Fpgid) {
 
 	/*
 	 * Size or print the process group ID.
 	 */
 	    if (!PrPass) {
-		(void) sprintf(buf, "%d", Lp->pgrp);
-		if ((len = strlen(buf)) > PgrpColW)
-		    PgrpColW = len;
+		(void) snpf(buf, sizeof(buf), "%d", Lp->pgid);
+		if ((len = strlen(buf)) > PgidColW)
+		    PgidColW = len;
 	    } else
-		(void) printf(" %*d", PgrpColW, Lp->pgrp);
+		(void) printf(" %*d", PgidColW, Lp->pgid);
 	}
 /*
  * Size or print the user ID or login name.
@@ -738,7 +781,7 @@ print_file()
  * Size or print the file descriptor, access mode and lock status.
  */
 	if (!PrPass) {
-	    (void) sprintf(buf, "%s%c%c",
+	    (void) snpf(buf, sizeof(buf), "%s%c%c",
 		Lf->fd,
 		(Lf->lock == ' ') ? Lf->access
 				  : (Lf->access == ' ') ? '-'
@@ -768,8 +811,11 @@ print_file()
  */
 
 	if (Fsv) {
+
+# if	!defined(HASNOFSADDR)
 	    if (Fsv & FSV_FA) {
-		cp =  (Lf->fsv & FSV_FA) ? print_kptr(Lf->fsa, buf) : "";
+		cp =  (Lf->fsv & FSV_FA) ? print_kptr(Lf->fsa, buf, sizeof(buf))
+					 : "";
 		if (!PrPass) {
 		    if ((len = strlen(cp)) > FsColW)
 			FsColW = len;
@@ -777,9 +823,12 @@ print_file()
 		    (void) printf(" %*.*s", FsColW, FsColW, cp);
 		    
 	    }
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSCOUNT)
 	    if (Fsv & FSV_CT) {
 		if (Lf->fsv & FSV_CT) {
-		    (void) sprintf(buf, "%ld", Lf->fct);
+		    (void) snpf(buf, sizeof(buf), "%ld", Lf->fct);
 		    cp = buf;
 		} else
 		    cp = "";
@@ -789,6 +838,9 @@ print_file()
 		} else
 		    (void) printf(" %*.*s", FcColW, FcColW, cp);
 	    }
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSFLAGS)
 	    if (Fsv & FSV_FG) {
 		if ((Lf->fsv & FSV_FG) && (FsvFlagX || Lf->ffg || Lf->pof))
 		    cp = print_fflags(Lf->ffg, Lf->pof);
@@ -800,14 +852,20 @@ print_file()
 		} else
 		    (void) printf(" %*.*s", FgColW, FgColW, cp);
 	    }
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 	    if (Fsv & FSV_NI) {
-		cp = (Lf->fsv & FSV_NI) ? print_kptr(Lf->fna, buf) : "";
+		cp = (Lf->fsv & FSV_NI) ? print_kptr(Lf->fna, buf, sizeof(buf))
+					: "";
 		if (!PrPass) {
 		    if ((len = strlen(cp)) > NiColW)
 			NiColW = len;
 		} else
 		    (void) printf(" %*.*s", NiColW, NiColW, cp);
 	    }
+# endif	/* !defined(HASNOFSNADDR) */
+
 	}
 #endif	/* defined(HASFSTRUCT) */
 
@@ -815,19 +873,28 @@ print_file()
  * Size or print the device information.
  */
 
-	if (Lf->dev_def) {
+	if (Lf->rdev_def) {
+	    dev = Lf->rdev;
+	    devs = 1;
+	} else if (Lf->dev_def) {
+	    dev = Lf->dev;
+	    devs = 1;
+	} else
+	    devs = 0;
+	if (devs) {
 
 #if	defined(HASPRINTDEV)
-	    cp = HASPRINTDEV(Lf);
+	    cp = HASPRINTDEV(Lf, &dev);
 #else	/* !defined(HASPRINTDEV) */
-	    (void) sprintf(buf, "%d,%d", major(Lf->dev), minor(Lf->dev));
+	    (void) snpf(buf, sizeof(buf), "%d,%d", GET_MAJ_DEV(dev),
+		GET_MIN_DEV(dev));
 	    cp = buf;
 #endif	/* defined(HASPRINTDEV) */
 
 	}
 
 	if (!PrPass) {
-	    if (Lf->dev_def)
+	    if (devs)
 		len = strlen(cp);
 	    else if (Lf->dev_ch)
 		len = strlen(Lf->dev_ch);
@@ -836,7 +903,7 @@ print_file()
 	    if (len > DevColW)
 		DevColW = len;
 	} else {
-	    if (Lf->dev_def)
+	    if (devs)
 		(void) printf(" %*.*s", DevColW, DevColW, cp);
 	    else {
 		if (Lf->dev_ch)
@@ -854,7 +921,7 @@ print_file()
 #if	defined(HASPRINTSZ)
 		cp = HASPRINTSZ(Lf);
 #else	/* !defined(HASPRINTSZ) */
-		(void) sprintf(buf, SzOffFmt_d, Lf->sz);
+		(void) snpf(buf, sizeof(buf), SzOffFmt_d, Lf->sz);
 		cp = buf;
 #endif	/* defined(HASPRINTSZ) */
 
@@ -864,7 +931,7 @@ print_file()
 #if	defined(HASPRINTOFF)
 		cp = HASPRINTOFF(Lf, 0);
 #else	/* !defined(HASPRINTOFF) */
-		(void) sprintf(buf, SzOffFmt_0t, Lf->off);
+		(void) snpf(buf, sizeof(buf), SzOffFmt_0t, Lf->off);
 		cp = buf;
 #endif	/* defined(HASPRINTOFF) */
 
@@ -874,7 +941,7 @@ print_file()
 #if	defined(HASPRINTOFF)
 		    cp = HASPRINTOFF(Lf, 1);
 #else	/* !defined(HASPRINTOFF) */
-		    (void) sprintf(buf, SzOffFmt_x, Lf->off);
+		    (void) snpf(buf, sizeof(buf), SzOffFmt_x, Lf->off);
 		    cp = buf;
 #endif	/* defined(HASPRINTOFF) */
 
@@ -899,7 +966,7 @@ print_file()
 #if	defined(HASPRINTOFF)
 		cp = HASPRINTOFF(Lf, 0);
 #else	/* !defined(HASPRINTOFF) */
-		(void) sprintf(buf, SzOffFmt_0t, Lf->off);
+		(void) snpf(buf, sizeof(buf), SzOffFmt_0t, Lf->off);
 		cp = buf;
 #endif	/* defined(HASPRINTOFF) */
 
@@ -908,7 +975,7 @@ print_file()
 #if	defined(HASPRINTOFF)
 		    cp = HASPRINTOFF(Lf, 1);
 #else	/* !defined(HASPRINTOFF) */
-		    (void) sprintf(buf, SzOffFmt_x, Lf->off);
+		    (void) snpf(buf, sizeof(buf), SzOffFmt_x, Lf->off);
 		    cp = buf;
 #endif	/* defined(HASPRINTOFF) */
 
@@ -922,7 +989,7 @@ print_file()
  */
 	if (Fnlink) {
 	    if (Lf->nlink_def) {
-		(void) sprintf(buf, " %ld", Lf->nlink);
+		(void) snpf(buf, sizeof(buf), " %ld", Lf->nlink);
 		cp = buf;
 	   } else
 		cp = "";
@@ -941,7 +1008,7 @@ print_file()
 #if	defined(HASPRINTINO)
 	    cp = HASPRINTINO(Lf);
 #else	/* !defined(HASPRINTINO) */
-	    (void) sprintf(buf, "%lu", Lf->inode);
+	    (void) snpf(buf, sizeof(buf), InodeFmt_d, Lf->inode);
 	    cp = buf;
 #endif	/* defined(HASPRINTINO) */
 
@@ -953,7 +1020,7 @@ print_file()
 		cp = "";
 	    break;
 	case 3:
-	    (void) sprintf(buf, "%#lx", Lf->inode);
+	    (void) snpf(buf, sizeof(buf), InodeFmt_x, Lf->inode);
 	    cp = buf;
 	    break;
 	default:
@@ -991,7 +1058,7 @@ printinaddr()
 {
 	int i, len, src;
 	char *host, *port;
-	int nl = MAXPATHLEN - 1;
+	int nl = Namechl - 1;
 	char *np = Namech;
 	char pbuf[32];
 /*
@@ -1012,10 +1079,11 @@ printinaddr()
 addr_too_long:
 
 		    {
-			(void) fputs("network addresses too long", stdout);
+			(void) snpf(Namech, Namechl,
+			    "network addresses too long");
 			return(1);
 		    }
-		(void) strcpy(np, "->");
+		(void) snpf(np, nl, "->");
 		np += 2;
 		nl -= 2;
 	    }
@@ -1080,7 +1148,7 @@ addr_too_long:
 			port = lkup_port(Lf->li[i].p, 1, src);
 		}
 		if (!port) {
-		    (void) sprintf(pbuf, "%d", Lf->li[i].p);
+		    (void) snpf(pbuf, sizeof(pbuf), "%d", Lf->li[i].p);
 		    port = pbuf;
 		}
 	    } else if (Lf->li[i].p == 0)
@@ -1092,7 +1160,7 @@ addr_too_long:
 		if ((len = strlen(host)) > nl)
 		    goto addr_too_long;
 		if (len) {
-		    (void) strcpy(np, host);
+		    (void) snpf(np, nl, "%s", host);
 		    np += len;
 		    nl -= len;
 		}
@@ -1103,10 +1171,9 @@ addr_too_long:
 	    if (port) {
 		if (((len = strlen(port)) + 1) >= nl)
 		    goto addr_too_long;
-		*np++ = ':';
-		(void) strcpy(np, port);
-		np += len;
-		nl -= (len + 1);
+		(void) snpf(np, nl, ":%s", port);
+		np += len + 1;
+		nl -= len - 1;
 	    }
 	}
 	if (Namech[0]) {
@@ -1132,7 +1199,7 @@ print_init()
 	    NlColW = strlen(NLTTL);
 	NmColW = strlen(NMTTL);
 	NodeColW = strlen(NODETTL);
-	PgrpColW = strlen(PGRPTTL);
+	PgidColW = strlen(PGIDTTL);
 	PidColW = strlen(PIDTTL);
 	PpidColW = strlen(PPIDTTL);
 	if (Fsize)
@@ -1145,15 +1212,33 @@ print_init()
 	UserColW = strlen(USERTTL);
 
 #if	defined(HASFSTRUCT)
-	FcColW = strlen(FCTTL);
-	FgColW = strlen(FGTTL);
+
+# if	!defined(HASNOFSADDR)
 	FsColW = strlen(FSTTL);
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSCOUNT)
+	FcColW = strlen(FCTTL);
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSFLAGS)
+	FgColW = strlen(FGTTL);
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 	NiColW = strlen(NiTtl);
+# endif	/* !defined(HASNOFSNADDR) */
 #endif	/* defined(HASFSTRUCT) */
 
+#if	defined(HASZONES)
+	if (Fzone)
+	    ZoneColW = strlen(ZONETTL);
+#endif	/* defined(HASZONES) */
+
 }
 
 
+#if	!defined(HASPRIVPRIPP)
 /*
  * printiproto() - print Internet protocol name
  */
@@ -1166,7 +1251,7 @@ printiproto(p)
 	static int m = -1;
 	char *s;
 
-	switch(p) {
+	switch (p) {
 
 #if	defined(IPPROTO_TCP)
 	case IPPROTO_TCP:
@@ -1794,18 +1879,19 @@ printiproto(p)
 	    s = (char *)NULL;
 	}
 	if (s)
-	    (void) sprintf(Lf->iproto, "%.*s", IPROTOL-1, s);
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, s);
 	else {	
 	    if (m < 0) {
 		for (i = 0, m = 1; i < IPROTOL-2; i++)
 		    m *= 10;
 	    }
 	    if (m > p)
-		(void) sprintf(Lf->iproto, "%d?", p);
+		(void) snpf(Lf->iproto, sizeof(Lf->iproto), "%d?", p);
 	    else
-		(void) sprintf(Lf->iproto, "*%d?", p % (m/10));
+		(void) snpf(Lf->iproto, sizeof(Lf->iproto), "*%d?", p % (m/10));
 	}
 }
+#endif	/* !defined(HASPRIVPRIPP) */
 
 
 /*
@@ -1819,7 +1905,7 @@ printname(nl)
 
 #if	defined(HASNCACHE)
 	char buf[MAXPATHLEN];
-	char *cp, *cp1;
+	char *cp;
 	int fp;
 #endif	/* defined(HASNCACHE) */
 
@@ -1845,26 +1931,17 @@ printname(nl)
 		ps++;
 	    goto print_nma;
 	}
-	if (Lf->ntype == N_CHR && Lf->dev_def && printchdevname(&Lf->dev, 0)) {
+	if (((Lf->ntype == N_BLK) || (Lf->ntype == N_CHR))
+	&&  Lf->dev_def && Lf->rdev_def
+	&&  printdevname(&Lf->dev, &Lf->rdev, 0, Lf->ntype))
+	{
 
 	/*
-	 * If this is a character device and it has a name, print it.
+	 * If this is a block or character device and it has a name, print it.
 	 */
 	    ps++;
 	    goto print_nma;
 	}
-
-#if	defined(HASBLKDEV)
-	if (Lf->ntype == N_BLK && Lf->dev_def && printbdevname(&Lf->dev, 0)) {
-
-	/*
-	 * If this is a block device and it has a name, print it.
-	 */
-	    ps++;
-	    goto print_nma;
-	}
-#endif	/* defined(HASBLKDEV) */
-
 	if (Lf->is_com) {
 
 	/*
@@ -1876,8 +1953,10 @@ printname(nl)
 	}
 
 #if	defined(HASPRIVNMCACHE)
-	if (HASPRIVNMCACHE(Lf))
+	if (HASPRIVNMCACHE(Lf)) {
+	    ps++;
 	    goto print_nma;
+	}
 #endif	/* defined(HASPRIVNMCACHE) */
 
 	if (Lf->lmi_srch) {
@@ -1906,28 +1985,87 @@ printname(nl)
 	 * Print the file system directory name, device name, and
 	 * possible path name components.
 	 */
+
+#if	!defined(HASNCACHE) || HASNCACHE<2
 	    if (Lf->fsdir) {
 		safestrprt(Lf->fsdir, stdout, 0);
 		ps++;
 	    }
+#endif	/* !defined(HASNCACHE) || HASNCACHE<2 */
 
 #if	defined(HASNCACHE)
-	    if (Lf->na
-	    &&  (cp = ncache_lookup(buf, sizeof(buf), &fp))) {
-		if (*cp == '\0')
+
+# if	HASNCACHE<2
+	    if (Lf->na) {
+		if (NcacheReload) {
+
+#  if	defined(NCACHELDPFX)
+		    NCACHELDPFX
+#  endif	/* defined(NCACHELDPFX) */
+
+		    (void) ncache_load();
+
+#  if	defined(NCACHELDSFX)
+		    NCACHELDSFX
+#  endif	/* defined(NCACHELDSFX) */
+
+		    NcacheReload = 0;
+		}
+		if ((cp = ncache_lookup(buf, sizeof(buf), &fp))) {
+		    char *cp1; 
+
+		    if (*cp == '\0')
+			goto print_nma;
+		    if (fp && Lf->fsdir) {
+			if (*cp != '/') {
+			    cp1 = strrchr(Lf->fsdir, '/');
+			    if (cp1 == (char *)NULL ||  *(cp1 + 1) != '\0')
+				putchar('/');
+			    }
+		    } else
+			(void) fputs(" -- ", stdout);
+		    safestrprt(cp, stdout, 0);
+		    ps++;
 		    goto print_nma;
-		if (fp && Lf->fsdir) {
-		    if (*cp != '/') {
-			cp1 = strrchr(Lf->fsdir, '/');
-			if (cp1 == (char *)NULL ||  *(cp1 + 1) != '\0')
-			    putchar('/');
+		}
+	    }
+# else	/* HASNCACHE>1 */
+	    if (NcacheReload) {
+
+#  if	defined(NCACHELDPFX)
+		    NCACHELDPFX
+#  endif	/* defined(NCACHELDPFX) */
+
+		(void) ncache_load();
+
+#  if	defined(NCACHELDSFX)
+		    NCACHELDSFX
+#  endif	/* defined(NCACHELDSFX) */
+
+		NcacheReload = 0;
+	    }
+	    if ((cp = ncache_lookup(buf, sizeof(buf), &fp))) {
+		if (fp) {
+		    safestrprt(cp, stdout, 0);
+		    ps++;
+		} else {
+		    if (Lf->fsdir) {
+			safestrprt(Lf->fsdir, stdout, 0);
+			ps++;
 		    }
-		} else
-		    (void) fputs(" -- ", stdout);
-		safestrprt(cp, stdout, 0);
-		ps++;
+		    if (*cp) {
+			(void) fputs(" -- ", stdout);
+			safestrprt(cp, stdout, 0);
+			ps++;
+		    }
+		}
 		goto print_nma;
 	    }
+	    if (Lf->fsdir) {
+		safestrprt(Lf->fsdir, stdout, 0);
+		ps++;
+	    }
+# endif	/* HASNCACHE<2 */
 #endif	/* defined(HASNCACHE) */
 
 	    if (Lf->fsdev) {
@@ -1960,11 +2098,11 @@ print_nma:
 	&&  (Lf->lts.type >= 0
 
 #if	defined(HASTCPTPIQ)
-	||   Lf->lts.rqs || Lf->lts.sqs
+	||   ((Ftcptpi & TCPTPI_QUEUES) && (Lf->lts.rqs || Lf->lts.sqs))
 #endif	/* defined(HASTCPTPIQ) */
 
 #if	defined(HASTCPTPIW)
-	||   Lf->lts.rws || Lf->lts.wws
+	||   ((Ftcptpi & TCPTPI_WINDOWS) && (Lf->lts.rws || Lf->lts.wws))
 #endif	/* defined(HASTCPTPIW) */
 
 	)) {
@@ -1986,7 +2124,11 @@ void
 printrawaddr(sa)
 	struct sockaddr *sa;		/* socket address */
 {
-	(void) sprintf(endnm(), "%u/%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u",
+	char *ep;
+	size_t sz;
+
+	ep = endnm(&sz);
+	(void) snpf(ep, sz, "%u/%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u",
 	    sa->sa_family,
 	    (unsigned char)sa->sa_data[0],
 	    (unsigned char)sa->sa_data[1],
@@ -2049,10 +2191,10 @@ printsockty(ty)
 #endif	/* defined(SOCK_SEQPACKET) */
 
 	default:
-	    (void) sprintf(buf, "SOCK_%#x", ty);
+	    (void) snpf(buf, sizeof(buf), "SOCK_%#x", ty);
 	    return(buf);
 	}
-	(void) sprintf(buf, "SOCK_%s", cp);
+	(void) snpf(buf, sizeof(buf), "SOCK_%s", cp);
 	return(buf);
 }
 
@@ -2062,8 +2204,8 @@ printsockty(ty)
  */
 
 char *
-printuid(u, ty)
-	UID_ARG u;			/* User IDentification number */
+printuid(uid, ty)
+	UID_ARG uid;			/* User IDentification number */
 	int *ty;			/* returned UID type pointer (NULL
 					 * (if none wanted).  If non-NULL
 					 * then: *ty = 0 = login name
@@ -2074,7 +2216,7 @@ printuid(u, ty)
 	struct stat sb;
 	static struct stat sbs;
 	static struct uidcache {
-	    uid_t u;
+	    uid_t uid;
 	    char nm[LOGINML+1];
 	    struct uidcache *next;
 	} **uc = (struct uidcache **)NULL;
@@ -2133,9 +2275,9 @@ printuid(u, ty)
 	/*
 	 * Search the UID cache.
 	 */
-	    i = (int)((((unsigned long)u * 31415L) >> 7) & (UIDCACHEL - 1));
+	    i = (int)((((unsigned long)uid * 31415L) >> 7) & (UIDCACHEL - 1));
 	    for (up = uc[i]; up; up = up->next) {
-		if (up->u == (uid_t)u) {
+		if (up->uid == (uid_t)uid) {
 		    if (ty)
 			*ty = 0;
 		    return(up->nm);
@@ -2146,10 +2288,10 @@ printuid(u, ty)
 	 *
 	 * Look up the login name from the UID for a new cache entry.
 	 */
-	    if (!(pw = getpwuid((uid_t)u))) {
+	    if (!(pw = getpwuid((uid_t)uid))) {
 		if (!Fwarn) {
 		    (void) fprintf(stderr, "%s: no pwd entry for UID %lu\n",
-			Pn, (unsigned long)u);
+			Pn, (unsigned long)uid);
 		}
 	    } else {
 
@@ -2160,12 +2302,12 @@ printuid(u, ty)
 		{
 		    (void) fprintf(stderr,
 			"%s: no space for UID cache entry for: %lu, %s)\n",
-			Pn, (unsigned long)u, pw->pw_name);
+			Pn, (unsigned long)uid, pw->pw_name);
 		    Exit(1);
 		}
 		(void) strncpy(upn->nm, pw->pw_name, LOGINML);
 		upn->nm[LOGINML] = '\0';
-		upn->u = (uid_t)u;
+		upn->uid = (uid_t)uid;
 		upn->next = uc[i];
 		uc[i] = upn;
 		if (ty)
@@ -2176,7 +2318,7 @@ printuid(u, ty)
 /*
  * Produce a numeric conversion of the UID.
  */
-	(void) sprintf(user, "%*lu", USERPRTL, (unsigned long)u);
+	(void) snpf(user, sizeof(user), "%*lu", USERPRTL, (unsigned long)uid);
 	if (ty)
 	    *ty = 1;
 	return(user);
@@ -2478,24 +2620,75 @@ printunkaf(fam, ty)
 	    break;
 #endif	/* defined(AF_OTS) */
 
+#if	defined(pseudo_AF_BLUE)
+	case pseudo_AF_BLUE:	/* packets for Blue box */
+	    p = "pseudo_";
+	    s = "BLUE";
+	    break;
+#endif	/* defined(pseudo_AF_BLUE) */
+
+#if	defined(AF_NDRV)	/* network driver raw access */
+	case AF_NDRV:
+	    s = "NDRV";
+	    break;
+#endif	/* defined(AF_NDRV) */
+
+#if	defined(AF_SYSTEM)	/* kernel event messages */
+	case AF_SYSTEM:
+	    s = "SYSTEM";
+	    break;
+#endif	/* defined(AF_SYSTEM) */
+
 #if	defined(AF_USER)
 	case AF_USER:
 	    s = "USER";
 	    break;
 #endif	/* defined(AF_USER) */
 
+#if	defined(pseudo_AF_KEY)
+	case pseudo_AF_KEY:
+	    p = "pseudo_";
+	    s = "KEY";
+	    break;
+#endif	/* defined(pseudo_AF_KEY) */
+
+#if	defined(AF_KEY)		/* Security Association DB socket */
+	case AF_KEY:			
+	    s = "KEY";
+	    break;
+#endif	/* defined(AF_KEY) */
+
+#if	defined(AF_NCA)		/* NCA socket */
+	case AF_NCA:			
+	    s = "NCA";
+	    break;
+#endif	/* defined(AF_NCA) */
+
+#if	defined(AF_POLICY)		/* Security Policy DB socket */
+	case AF_POLICY:
+	    s = "POLICY";
+	    break;
+#endif	/* defined(AF_POLICY) */
+
+#if	defined(AF_PPP)		/* PPP socket */
+	case AF_PPP:			
+	    s = "PPP";
+	    break;
+#endif	/* defined(AF_PPP) */
+
 	default:
 	    if (!ty)
-		(void) sprintf(Namech, "%#x", fam);
+		(void) snpf(Namech, Namechl, "%#x", fam);
 	    else
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "no further information on family %#x", fam);
 	    return;
 	}
 	if (!ty)
-	    (void) sprintf(Namech, "%sAF_%s", p, s);
+	    (void) snpf(Namech, Namechl, "%sAF_%s", p, s);
 	else
-	    (void) sprintf(Namech, "no further information on %sAF_%s", p, s);
+	    (void) snpf(Namech, Namechl, "no further information on %sAF_%s",
+		p, s);
 	return;
 }
 
@@ -2526,7 +2719,7 @@ update_portmap(pt, pn)
 		Pn, (int)(nl + 1), pn, pt->name);
 	    Exit(1);
 	}
-	(void) sprintf(cp, "%s[%s]", pn, pt->name);
+	(void) snpf(cp, nl + 1, "%s[%s]", pn, pt->name);
 	(void) free((FREE_P *)pt->name);
 	pt->name = cp;
 	pt->nl = nl;
